/*
 * motor.h
 *
 *  Created on: 02.02.2014
 *      Author: AndreR
 */

#ifndef MOTOR_H_
#define MOTOR_H_

#include <stdint.h>

#include "stm32f30x.h"
#include "intercom_constants.h"
#include "util/flash_fs.h"
#include "util/ema_filter.h"
#include "util/pid.h"
#include "util/lag_element.h"
#include "ch.h"

#define IN_CCM __attribute__((section(".ccmtext")))
#define IN_CCM_VAR __attribute__((section(".ccmvar")))

#define DRIVE_ADC_SAMPLES 16
#define DRIVE_MEAS_SAMPLES 100
#define DRIVE_MOVE_THRESHOLD 100

#define MOTOR_KN 3760.0f			// [RPM/Volt]
#define MOTOR_KE (1.0f/MOTOR_KN)	// [V/RPM]
#define MOTOR_R 0.403f				// [Ohm]
#define MOTOR_KM 0.00254f			// [Nm/A]

#define MOTOR_MODE_DISABLE	0
#define MOTOR_MODE_BLOCK	1
#define MOTOR_MODE_SVPWM	2
#define MOTOR_MODE_TEST1	3
#define MOTOR_MODE_TEST2	4
#define MOTOR_MODE_TEST3	5

#define MOTOR_CTRL_MODE_NONE		0
#define MOTOR_CTRL_MODE_PID_SPEED	1

struct _Motor;

typedef struct _Motor
{
	volatile uint8_t halPos;
	volatile float curVelocity;	// current velocity [rounds per minute]

	volatile uint8_t mode;
	volatile uint8_t ctrlMode;

	float rmsCurrent;
	volatile int32_t outLevel;	// [-3600 - 3600]

	float setVoltage;
	uint32_t setVoltageTimestamp;

	PID pidSpeed;

	// ### Setup ###
	// ADC calibration values
	uint32_t adcCal[2];

	// ### Measured values ###
	// ADC DMA storage area
	union
	{
		uint16_t adc[2][DRIVE_ADC_SAMPLES];
		uint32_t adc32[2][DRIVE_ADC_SAMPLES/2];
	};

	EMAFilter biasCurrent;

	LagElementPT1 tempPT1;
	float temp; // [K] over environment
	uint8_t overheated;
} Motor;

extern Motor motor;
extern Motor* pMotor;

void MotorInit();

// Low-level functions
void MotorSetMode(uint8_t mode);
void MotorSetVoltage(float vol);

// Select controller
void MotorSetCtrlMode(uint8_t mode);

// Speed controller
void MotorSpeedSetPIDParams(float p, float i, float d);
void MotorSpeedSetVelocity(float rpm);

// SVPWM helper functions (mostly for tests)
void MotorSetPhaseVoltagesSVPWM(float Va, float Vb, float Vc);
void MotorSetAlphaBeta(float Valpha, float Vbeta);
void MotorSetVoltageAngle(float Vref, float angle);

#endif /* MOTOR_H_ */
