/*
 * io.c
 *
 *  Created on: 24.12.2014
 *      Author: AndreR
 */

#include "io.h"
#include "util/intercom.h"
#include "intercom_constants.h"
#include "hal/motor.h"
#include "supply.h"
#include "hal/kicker.h"
#include "kick.h"

#define KICKER_MODE_FORCE		0
#define KICKER_MODE_ARM			1
#define KICKER_MODE_DISARM		2
#define KICKER_MODE_DRIBBLER	3
#define KICKER_MODE_IGNORE		9

IOData io;

static void ctrlCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)dataLength;

	if(io.ignoreMainCmds)
		return;

	switch(pHeader->command)
	{
		case IC_KD_MATCH_COMMAND:
		{
			if(chVTTimeElapsedSinceX(io.ignoreStartTime) < MS2ST(5000))
				return;
			else
				io.ignoreStartTime = chVTGetSystemTime()-MS2ST(5500);

			KdCommand* pCmd = (KdCommand*)pData;

			// Dribbler
			if((pCmd->dribblerSpeed > DRIVE_MOVE_THRESHOLD || pCmd->dribblerSpeed < -DRIVE_MOVE_THRESHOLD) && supply.batEmpty == 0)
			{
				MotorSpeedSetVelocity(pCmd->dribblerSpeed);
			}
			else
			{
				MotorSpeedSetVelocity(0);
			}

			// Kicker
			if(pCmd->autoChg && supply.batEmpty == 0)
			{
				KickerEnableAutoRecharge(1);
				KickerEnableCharge(1);
			}
			else
			{
				KickDischarge();
			}

			switch(pCmd->mode)
			{
				case KICKER_MODE_FORCE: KickFireSpeed(pCmd->speed, pCmd->device); break;
				case KICKER_MODE_ARM: KickArmIr(pCmd->speed, pCmd->device); break;
				case KICKER_MODE_DRIBBLER: KickArmDribbler(pCmd->speed, pCmd->device); break;
				case KICKER_MODE_IGNORE: { KickDisarm(); io.ignoreStartTime = chVTGetSystemTime(); } break;
				default: KickDisarm(); break;
			}
		}
		break;
		case IC_KD_FIRE:
		{
			KdFire* pFire = (KdFire*)pData;

			switch(pFire->mode)
			{
				case KICKER_MODE_FORCE: KickFireSpeed(pFire->speed, pFire->device); break;
				case KICKER_MODE_ARM: KickArmIr(pFire->speed, pFire->device); break;
				default: KickDisarm(); break;
			}
		}
		break;
	}
}

void IoInit()
{
	IntercomClient iClient;
	iClient.pUserData = 0;
	iClient.rxCallback = &ctrlCallback;
	iClient.section = INTERCOM_SECTION_KD;

	IntercomAddClient(iClient);

	io.ignoreStartTime = -MS2ST(5500);
}

void IoTask(void* params)
{
	(void)params;

	uint32_t lastPowerGoodTime = chVTGetSystemTime();
	IntercomHeader iHeader;
	KdFeedback kdFeedback;

	iHeader.address = INTERCOM_ADDRESS_MAIN | INTERCOM_ADDRESS_MEDIA;
	iHeader.section = INTERCOM_SECTION_KD;
	iHeader.command = IC_KD_FEEDBACK;

	chRegSetThreadName("I/O");

	while(1)
	{
		chThdSleepMilliseconds(5);

		if(supply.bat > 10.5f)
			lastPowerGoodTime = chVTGetSystemTime();

		if(chVTTimeElapsedSinceX(lastPowerGoodTime) > S2ST(10) && supply.batEmpty == 0)
		{
			supply.batEmpty = 1;

			KickDischarge();
		}

		kdFeedback.kicker.cap = kicker.vCap;
		kdFeedback.kicker.chg = kicker.iChg;
		kdFeedback.kicker.state = kicker.state | (kicker.autoCharge << 8) | (kicker.autoDischarge << 12);
		kdFeedback.kicker.kickCounter = kicker.kickCounter;

		kdFeedback.dribbler.velocity = motor.curVelocity;
		kdFeedback.dribbler.temp = motor.temp;
		kdFeedback.dribbler.overheated = motor.overheated;

		if(supply.irResult[0] > supply.irResult[1])
		{
			kdFeedback.barrier.level[0] = supply.irResult[0];
			kdFeedback.barrier.level[1] = supply.irResult[1];
		}
		else
		{
			kdFeedback.barrier.level[0] = supply.irResult[1];
			kdFeedback.barrier.level[1] = supply.irResult[0];
		}
		kdFeedback.barrier.interrupted = kick.interrupted;

		kdFeedback.power.bat = supply.bat;
		kdFeedback.power.cur = supply.shunt;
		kdFeedback.power.batEmpty = supply.batEmpty;

//		kdFeedback.config.maxCapLevel = kick.config.maxVoltage;
//		kdFeedback.config.cooldown = kick.config.cooldown;
//		kdFeedback.config.irThreshold = kick.config.irThreshold;
//		kdFeedback.config.irFilterLimit = kick.config.irFilterThreshold;

		IntercomSend(&iHeader, sizeof(KdFeedback), &kdFeedback);
	}
}
