/*
 * test_drive.c
 *
 *  Created on: 14.12.2015
 *      Author: AndreR <andre@ryll.cc>
 */

#include "test_drive.h"
#include "styles.h"
#include "util/intercom.h"
#include "util/intercom_reliable.h"
#include "../test.h"
#include <stdio.h>
#include <math.h>

static const char* pMotors[] = {"FR", "FL", "RL", "RR", "DB"};

static struct _handles
{
	GHandle hOverviewCont;
	GHandle hBtnTestAll;

	struct
	{
		GHandle hBtnMotor;
		GHandle hMotCont;
		GLabelObject* hRes;
		char res[8];
		GLabelObject* hEMF;
		char emf[8];
		GLabelObject* hCur;
		char cur[8];
		GLabelObject* hVel;
		char vel[8];
	} mot[5];
} handles;

static int16_t eventHandler(GEvent* pEvent)
{
	if(pEvent->type == GEVENT_GWIN_BUTTON)
	{
		GEventGWinButton* pBtn = (GEventGWinButton*)pEvent;

		if(pBtn->gwin == handles.hBtnTestAll)
		{
			for(uint8_t i = 0; i < 5; i++)
			{
				handles.mot[i].hRes->w.pstyle = &BlackWidgetStyle;
				gwinSetText(&handles.mot[i].hRes->w.g, "-", FALSE);

				handles.mot[i].hEMF->w.pstyle = &BlackWidgetStyle;
				gwinSetText(&handles.mot[i].hEMF->w.g, "-", FALSE);

				handles.mot[i].hCur->w.pstyle = &BlackWidgetStyle;
				gwinSetText(&handles.mot[i].hCur->w.g, "-", FALSE);

				handles.mot[i].hVel->w.pstyle = &BlackWidgetStyle;
				gwinSetText(&handles.mot[i].hVel->w.g, "-", FALSE);

				gwinSetEnabled(handles.mot[i].hBtnMotor, FALSE);

				TestScheduleMotorTest(IC_TEST_MOT_DYN, i);
			}

			gwinSetEnabled(handles.hBtnTestAll, FALSE);
		}

		for(uint8_t i = 0; i < 5; i++)
		{
			if(pBtn->gwin == handles.mot[i].hBtnMotor)
			{
				TestScheduleMotorTest(IC_TEST_MOT_DYN, i);

				gwinSetEnabled(handles.mot[i].hBtnMotor, FALSE);
			}
		}
	}

	return -1;
}

void TestDriveProgress(TestProgress* pProgress)
{
	static char buf[8];

	snprintf(buf, 8, "%3hu%%", (uint16_t)pProgress->percentComplete);

	switch(pProgress->testId)
	{
		case IC_TEST_MOT_DYN:
		{
			gwinSetText(&handles.mot[pProgress->motorId].hRes->w.g, buf, FALSE);
		}
		break;
	}
}

static void reasoningDriveMotor(TestMotorDynamicsResult* pResult)
{
	if(pResult->R < 0.3f || pResult->R > 0.6f)
	{
		handles.mot[pResult->motorId].hRes->w.pstyle = &YellowTextStyle;
	}

	if(pResult->R < 0.2f || pResult->R > 0.7f)
	{
		handles.mot[pResult->motorId].hRes->w.pstyle = &RedTextStyle;
	}

	if(pResult->k < 0.023f || pResult->k > 0.03f)
	{
		handles.mot[pResult->motorId].hEMF->w.pstyle = &RedTextStyle;
	}

	if(pResult->cur < 0.06f || pResult->cur > 0.1f)
	{
		handles.mot[pResult->motorId].hCur->w.pstyle = &YellowTextStyle;
	}

	if(pResult->cur < 0.03f || pResult->cur > 0.2f)
	{
		handles.mot[pResult->motorId].hCur->w.pstyle = &RedTextStyle;
	}

	if(pResult->speed < 210 || pResult->speed > 240)
	{
		handles.mot[pResult->motorId].hVel->w.pstyle = &YellowTextStyle;
	}

	if(pResult->speed < 150 || pResult->speed > 300)
	{
		handles.mot[pResult->motorId].hVel->w.pstyle = &RedTextStyle;
	}
}

static void reasoningDribblerMotor(TestMotorDynamicsResult* pResult)
{
	if(pResult->R < 0.8f || pResult->R > 1.1f)
	{
		handles.mot[pResult->motorId].hRes->w.pstyle = &YellowTextStyle;
	}

	if(pResult->R < 0.6f || pResult->R > 1.3f)
	{
		handles.mot[pResult->motorId].hRes->w.pstyle = &RedTextStyle;
	}

	if(pResult->k < 0.008f || pResult->k > 0.012f)
	{
		handles.mot[pResult->motorId].hEMF->w.pstyle = &RedTextStyle;
	}

	if(pResult->cur < 0.20f || pResult->cur > 0.30f)
	{
		handles.mot[pResult->motorId].hCur->w.pstyle = &YellowTextStyle;
	}

	if(pResult->cur < 0.1f || pResult->cur > 0.5f)
	{
		handles.mot[pResult->motorId].hCur->w.pstyle = &RedTextStyle;
	}

	if(pResult->speed < 620 || pResult->speed > 660)
	{
		handles.mot[pResult->motorId].hVel->w.pstyle = &YellowTextStyle;
	}

	if(pResult->speed < 580 || pResult->speed > 700)
	{
		handles.mot[pResult->motorId].hVel->w.pstyle = &RedTextStyle;
	}
}

void TestDriveDynamicResult(TestMotorDynamicsResult* pResult)
{
	snprintf(handles.mot[pResult->motorId].res, 8, "%.3f", pResult->R);
	snprintf(handles.mot[pResult->motorId].emf, 8, "%.3f", pResult->k);
	snprintf(handles.mot[pResult->motorId].cur, 8, "%.3f", pResult->cur);
	snprintf(handles.mot[pResult->motorId].vel, 8, "%.3f", pResult->speed);

	gwinSetText(&handles.mot[pResult->motorId].hRes->w.g, handles.mot[pResult->motorId].res, FALSE);
	gwinSetText(&handles.mot[pResult->motorId].hEMF->w.g, handles.mot[pResult->motorId].emf, FALSE);
	gwinSetText(&handles.mot[pResult->motorId].hCur->w.g, handles.mot[pResult->motorId].cur, FALSE);
	gwinSetText(&handles.mot[pResult->motorId].hVel->w.g, handles.mot[pResult->motorId].vel, FALSE);

	handles.mot[pResult->motorId].hRes->w.pstyle = &GreenTextStyle;
	handles.mot[pResult->motorId].hEMF->w.pstyle = &GreenTextStyle;
	handles.mot[pResult->motorId].hCur->w.pstyle = &GreenTextStyle;
	handles.mot[pResult->motorId].hVel->w.pstyle = &GreenTextStyle;

	if(pResult->motorId > 3)
		reasoningDribblerMotor(pResult);
	else
		reasoningDriveMotor(pResult);

	gwinSetEnabled(handles.mot[pResult->motorId].hBtnMotor, TRUE);

	if(pResult->motorId > 3)
		gwinSetEnabled(handles.hBtnTestAll, TRUE);
}

GHandle TestDriveCreate()
{
    GWidgetInit wi = { { 0, 60, 240, 260, FALSE, 0 }, "Drives", 0, &eventHandler, 0 };
    GHandle hTop = gwinContainerCreate(0, &wi, 0);

    // Container for current settings
    GWidgetInit overviewContInit = { { 0, 0, 240, 260, TRUE, hTop }, 0, 0, 0, 0 };
    handles.hOverviewCont = gwinContainerCreate(0, &overviewContInit, 0);

    for(uint8_t i = 0; i < 5; i++)
    {
		GWidgetInit motBtnInit = { { 30+i*40, 5, 40, 40, TRUE, handles.hOverviewCont }, pMotors[i], 0, 0, 0 };
		handles.mot[i].hBtnMotor = gwinButtonCreate(0, &motBtnInit);

	    GWidgetInit resInit = { { 25+i*40, 55, 35, 35, TRUE, handles.hOverviewCont }, " -", 0, 0, 0 };
	    handles.mot[i].hRes = (GLabelObject*)gwinLabelCreate(0, &resInit);

	    GWidgetInit emfInit = { { 25+i*40, 95, 35, 35, TRUE, handles.hOverviewCont }, " -", 0, 0, 0 };
	    handles.mot[i].hEMF = (GLabelObject*)gwinLabelCreate(0, &emfInit);

	    GWidgetInit curInit = { { 25+i*40, 135, 35, 35, TRUE, handles.hOverviewCont }, " -", 0, 0, 0 };
	    handles.mot[i].hCur = (GLabelObject*)gwinLabelCreate(0, &curInit);

	    GWidgetInit velInit = { { 25+i*40, 175, 35, 35, TRUE, handles.hOverviewCont }, " -", 0, 0, 0 };
	    handles.mot[i].hVel = (GLabelObject*)gwinLabelCreate(0, &velInit);
    }

    GWidgetInit resLabelInit = { { 5, 55, 55, 16, TRUE, handles.hOverviewCont }, "Res", 0, 0, 0 };
    gwinLabelCreate(0, &resLabelInit);
    GWidgetInit resLabelInit2 = { { 5, 71, 55, 16, TRUE, handles.hOverviewCont }, "??", 0, 0, 0 };
    gwinLabelCreate(0, &resLabelInit2);

    GWidgetInit emfLabelInit = { { 5, 95, 55, 16, TRUE, handles.hOverviewCont }, "EMF", 0, 0, 0 };
    gwinLabelCreate(0, &emfLabelInit);
    GWidgetInit emfLabelInit2 = { { 5, 111, 55, 16, TRUE, handles.hOverviewCont }, "0.027", 0, 0, 0 };
    gwinLabelCreate(0, &emfLabelInit2);

    GWidgetInit curLabelInit = { { 5, 135, 55, 16, TRUE, handles.hOverviewCont }, "Cur", 0, 0, 0 };
    gwinLabelCreate(0, &curLabelInit);
    GWidgetInit curLabelInit2 = { { 5, 151, 55, 16, TRUE, handles.hOverviewCont }, "0.13", 0, 0, 0 };
    gwinLabelCreate(0, &curLabelInit2);

    GWidgetInit velLabelInit = { { 5, 175, 55, 16, TRUE, handles.hOverviewCont }, "Vel", 0, 0, 0 };
    gwinLabelCreate(0, &velLabelInit);
    GWidgetInit velLabelInit2 = { { 5, 191, 55, 16, TRUE, handles.hOverviewCont }, "302", 0, 0, 0 };
    gwinLabelCreate(0, &velLabelInit2);

	GWidgetInit testAllBtnInit = { { 5, 220, 230, 40, TRUE, handles.hOverviewCont }, "Test All", 0, 0, 0 };
	handles.hBtnTestAll = gwinButtonCreate(0, &testAllBtnInit);

    return hTop;
}
