/*
 * kicker.h
 *
 *  Created on: 17.11.2015
 *      Author: AndreR
 */

#ifndef KICKER_H_
#define KICKER_H_

#include "ch.h"
#include "util/ema_filter.h"
#include "util/config.h"
#include "commands.h"

#define KICKER_DEVICE_STRAIGHT	0
#define KICKER_DEVICE_CHIP		1

#define KICKER_EVENT_QUEUE_SIZE 10

typedef struct PACKED _KickerInternalConfig
{
	float maxVoltage;		// [V]
	float chgHysteresis;	// [V]
	uint32_t cooldown;		// [ms]
	float irThreshold;		// [V]
	float kickVoltageLimit; // ignore kick cmds below kickVoltageLimit*curVoltage
	float aimTolerance;		// [rad]

	float kickOffset;
	float kickFactor;
	float kickMax;

	float chipOffset;
	float chipFactor;
	float chipMax;
} KickerInternalConfig;


typedef struct _Kicker
{
	uint8_t installed;

	float vCap;
	float iChg;

	float chgOffset;

	uint8_t autoCharge;
	uint8_t state;

	uint8_t autoDischarge;

	uint8_t interrupted;

	float vIrOn;
	float vIrOff;

	float stress;

	uint32_t lastKickDone;

	uint8_t kickCounter;

	mailbox_t eventQueue;
	msg_t eventQueueData[KICKER_EVENT_QUEUE_SIZE];

	EMAFilter chgOffsetFilter;

	EMAFilter capAvg;

	KickerInternalConfig config;
	ConfigFile* pConfigFile;

	struct _armed
	{
		uint8_t isActive;
		uint8_t device;
		float speed;

		uint8_t aim;
	} armed;

	uint32_t lastCapChgUpdate;
} Kicker;

extern Kicker kicker;

void KickerInit();
void KickerTask(void* params);
void KickerCapChgUpdate(float vCap, float iChg);
void KickerIrUpdate(float vIrOn, float vIrOff);

void KickerFireSpeed(float speed, uint8_t device);
void KickerFire(float duration, uint8_t device);
void KickerArm(float speed, uint8_t device);
void KickerAimArm(float speed, uint8_t device);
void KickerDisarm();

void KickerEnableAutoRecharge(uint8_t enable);
void KickerEnableCharge(uint8_t enable);
void KickerAutoDischarge();

void KickerSetMaxVoltage(float vMax);
void KickerSetCooldown(uint32_t cooldown);

#endif /* KICKER_H_ */
