/*
 * motors.h
 *
 *  Created on: 02.11.2015
 *      Author: AndreR
 */

#ifndef MOTORS_H_
#define MOTORS_H_

#include "ch.h"
#include "util/abg_filter.h"
#include "util/ema_filter.h"
#include "util/pid.h"

#define MOTORS_DRIVE_MODE_OFF		0
#define MOTORS_DRIVE_MODE_VELOCITY	1
#define MOTORS_DRIVE_MODE_VOLTAGE	2

#define MOTORS_DRIBBLER_BAR_RPM_TO_MOTOR_RADS 0.05235987755983f // (1/60.0f*1/2.0f*2.0f*M_PI)
#define MOTORS_DRIBBLER_MOTOR_RADS_TO_BAR_RPM 19.09859317102744f

typedef struct _Motors
{
	uint8_t driveMode;
	uint8_t acceptCtrlOutput;

	struct _drive
	{
		GPIO_TypeDef* pFaultPort;
		uint16_t ff1;
		uint16_t ff2;
		uint8_t faultStatus;

		GPIO_TypeDef* pDirPort;
		uint16_t dirPin;

		volatile uint32_t* pCCR;
		volatile uint32_t* pEncCNT;

		ABGFilter abg;
		float velocity; // motor rad/s
		uint32_t velocityTimestamp;

		PID pid;

		float setVelocity;	// motor rad/s
		float setVoltage;	// set by user, activated by DRIVE_MODE_VOLTAGE

		float outVoltage;	// voltage applied to motor by any control method
		uint32_t outVoltageTimestamp;
	} drive[4];

	struct _dribbler
	{
		uint8_t enableCtrl;

		float velocity;	// motor rad/s
		systime_t lastUpdateTime;

		EMAFilter ema;

		PID pid;
		float setVelocity;	// motor rad/s
	} dribbler;
} Motors;

extern Motors motors;

void MotorsInit();
void MotorsDriveEnable(uint8_t enable);
void MotorsDribblerEnable(uint8_t enable);
void MotorsDriveSetRaw(uint16_t ccr[4]);
void MotorsDrivePrintFault();
void MotorsDribblerSetRaw(uint16_t ccr);
void MotorsDribblerSetVoltage(float vol);
void MotorsDribblerSetVelocity(float velRPM);

void MotorsDriveSetVoltage(uint8_t motor, float vol);
void MotorsDrivePrintEncoders();
void MotorsDriveTask(void* params);

#endif /* MOTORS_H_ */
