/*
 * network_impl.h
 *
 *  Created on: 01.07.2010
 *      Author: AndreR
 */

#ifndef __NETWORK_IMPL_H__
#define __NETWORK_IMPL_H__

#include "util/nrf24_ex.h"
#include "util/flash_fs.h"
#include "util/fifo_lin.h"
#include "util/config.h"

#include "constants.h"
#include "commands.h"
#include "intercom_constants.h"

#include "ch.h"

typedef void(*PosCallback)();

#define NETWORK_REL_EVENT_SIZE 4

typedef struct _NetworkImpl
{
	FifoLin reliableCmds;
	uint16_t seq;
	msg_t relEventsData[NETWORK_REL_EVENT_SIZE];
	mailbox_t relEvents;

	binary_semaphore_t timerSem;
	mutex_t sendPacketMutex;
	mutex_t printMutex;

	NRF24Queue queue;
	NRF24Ex nrf24;

	uint32_t lastUSec;
	uint32_t avgRxPeriod;

	PosCallback posCb;

	int16_t lastVisionPos[3];
	uint32_t lastVisionPosTimestamp;
	uint32_t lastVisionDelay;

	ConfigNetwork config;
	ConfigFile* pNetworkConfig;

	NRF24ReadWriteDelay rwDelay;
	ConfigFile* pRwDelayConfig;

	uint8_t botDisabled;
	uint32_t lastMatchCtrlTime;

	uint32_t lastUIWifiStatTime;

	uint8_t sumatraOnline;
	uint8_t bsOnline;

	uint8_t skillOverride;

	PresenterWifiStat wifiStats;
} NetworkImpl;

extern NetworkImpl networkImpl;

void	NetworkInit();
void	NetworkTask(void* params);
void	NetworkReliableTask(void* params);
int16_t NetworkSendPacket(PacketHeader* pHeader, void* _pData, uint16_t dataLength);
void	NetworkSendPacketRaw(void* _pData, uint16_t dataLength);
int16_t NetworkSendPacketReliable(PacketHeader* pHeader, void* _pData, uint16_t dataLength);
void	NetworkSetPosCallback(PosCallback cb);
void	NetworkSaveConfig();
void	NetworkSendAnnouncement(uint32_t id);

#endif /* __NETWORK_IMPL_H__ */
