/*
 * main_status.c
 *
 *  Created on: 05.10.2015
 *      Author: AndreR
 */

#include "main_status.h"
#include "bot_cover.h"
#include "ids.h"
#include "util/intercom_reliable.h"
#include "styles.h"
#include <stdio.h>

static struct _handles
{
	GHandle hBotCover;
	GHandle hDischarge;
	GHandle hBatProg;
	GHandle hCur;
	GHandle hChannel;
	GHandle hSpeed;
	GHandle hBarrier;
	GHandle hHwId;
	GHandle hCapProg;
	GHandle hChg;
	GHandle hCharged;

	GTimer flashTimer;
} handles;

static void flashFunc(void* param)
{
	(void)param;

	gwinSetVisible(handles.hCharged, !gwinGetVisible(handles.hCharged));
}

static int16_t eventHandler(GEvent* pEvent)
{
	if(pEvent->type == GEVENT_GWIN_BUTTON)
	{
		GEventGWinButton* pBtn = (GEventGWinButton*)pEvent;

		if(pBtn->gwin == handles.hBotCover)
		{
			return GUI_WINDOW_BOT_ID;
		}

		if(pBtn->gwin == handles.hDischarge)
		{
			IntercomHeader iHeader;
			iHeader.address = INTERCOM_ADDRESS_KD;
			iHeader.section = INTERCOM_SECTION_KD;
			iHeader.command = IC_KD_MATCH_COMMAND;

			KdCommand kdCmd = {0, 9, 0, 0, 0};

			IntercomReliableSend(&iHeader, sizeof(KdCommand), &kdCmd);
		}
	}

	return -1;
}

GHandle MainStatusCreate()
{
	gtimerInit(&handles.flashTimer);

    GWidgetInit wi = { { 0, 60, 240, 260, FALSE, 0 }, "Status", 0, &eventHandler, 0 };
    GHandle hTop = gwinContainerCreate(0, &wi, 0);

    // Power
    GWidgetInit powerContInit = { { 0, 0, 240, 45, TRUE, hTop }, 0, 0, 0, 0 };
    GHandle hPowerCont = gwinContainerCreate(0, &powerContInit, 0);

    GWidgetInit batLabelInit = { { 5, 5, 70, 15, TRUE, hPowerCont }, "Battery:", 0, 0, 0 };
    gwinLabelCreate(0, &batLabelInit);

    GWidgetInit batInit = { { 5, 25, 110, 15, TRUE, hPowerCont }, 0, 0, 0, 0 };
    handles.hBatProg = gwinProgressbarCreate(0, &batInit);
    gwinProgressbarSetPosition(handles.hBatProg, 50);
    gwinSetText(handles.hBatProg, "11.1V", FALSE);

    GWidgetInit curLabelInit = { { 125, 5, 90, 15, TRUE, hPowerCont }, "Usage:", 0, 0, 0 };
    gwinLabelCreate(0, &curLabelInit);

    GWidgetInit curInit = { { 125, 25, 60, 15, TRUE, hPowerCont }, "4.2W", 0, 0, 0 };
    handles.hCur = gwinLabelCreate(0, &curInit);

    // Wifi: Channel, speed, ID
    GWidgetInit miscContInit = { { 0, 50, 240, 85, TRUE, hTop }, 0, 0, 0, 0 };
    GHandle hMiscCont = gwinContainerCreate(0, &miscContInit, 0);

	GWidgetInit channelLabelInit = { { 5, 5, 70, 15, TRUE, hMiscCont }, "Channel:", 0, 0, 0 };
	gwinLabelCreate(0, &channelLabelInit);
	GWidgetInit channelInit = { { 5, 25, 70, 15, TRUE, hMiscCont }, "0", 0, 0, 0 };
	handles.hChannel = gwinLabelCreate(0, &channelInit);

	GWidgetInit speedLabelInit = { { 85, 5, 70, 15, TRUE, hMiscCont }, "Speed:", 0, 0, 0 };
	gwinLabelCreate(0, &speedLabelInit);
	GWidgetInit speedInit = { { 85, 25, 70, 15, TRUE, hMiscCont }, "2MBit/s", 0, 0, 0 };
	handles.hSpeed = gwinLabelCreate(0, &speedInit);

	GWidgetInit idLabelInit = { { 165, 5, 70, 15, TRUE, hMiscCont }, "ID:", 0, 0, 0 };
	gwinLabelCreate(0, &idLabelInit);
	handles.hBotCover = BotCoverCreate(165, 25, hMiscCont, 11, 0);

	GWidgetInit barrierLabelInit = { { 5, 45, 70, 15, TRUE, hMiscCont }, "Barrier:", 0, 0, 0 };
	gwinLabelCreate(0, &barrierLabelInit);
	GWidgetInit barrierInit = { { 5, 65, 70, 15, TRUE, hMiscCont }, "0/0", 0, 0, 0 };
	handles.hBarrier = gwinLabelCreate(0, &barrierInit);

	GWidgetInit hwIdLabelInit = { { 85, 45, 70, 15, TRUE, hMiscCont }, "HW ID:", 0, 0, 0 };
	gwinLabelCreate(0, &hwIdLabelInit);
	GWidgetInit hwIdInit = { { 85, 65, 70, 15, TRUE, hMiscCont }, "255", 0, 0, 0 };
	handles.hHwId = gwinLabelCreate(0, &hwIdInit);

    // Kicker level
	GWidgetInit kickerContInit = { { 0, 140, 240, 120, TRUE, hTop }, 0, 0, 0, 0 };
	GHandle hKickerCont = gwinContainerCreate(0, &kickerContInit, 0);

	GWidgetInit capLabelInit = { { 5, 5, 110, 15, TRUE, hKickerCont }, "Kicker Level:", 0, 0, 0 };
	gwinLabelCreate(0, &capLabelInit);
	GWidgetInit capInit = { { 5, 25, 145, 15, TRUE, hKickerCont }, 0, 0, 0, 0 };
	handles.hCapProg = gwinProgressbarCreate(0, &capInit);
	gwinProgressbarSetRange(handles.hCapProg, 0, 180);
	gwinProgressbarSetPosition(handles.hCapProg, 0);
	gwinSetText(handles.hCapProg, "0.0V", FALSE);

	GWidgetInit chgLabelInit = { { 160, 5, 75, 15, TRUE, hKickerCont }, "Chg Amps:", 0, 0, 0 };
	gwinLabelCreate(0, &chgLabelInit);
	GWidgetInit chgInit = { { 160, 25, 75, 15, TRUE, hKickerCont }, "0.0A", 0, 0, 0 };
	handles.hChg = gwinLabelCreate(0, &chgInit);

	GWidgetInit chargedInit = { { 20, 50, 200, 20, FALSE, hKickerCont }, ">>> Kicker charged! <<<", 0, 0, 0 };
	handles.hCharged = gwinLabelCreate(0, &chargedInit);
	((GWidgetObject*)handles.hCharged)->pstyle = &RedTextStyle;
	gwinSetFont(handles.hCharged, gdispOpenFont("DejaVuSans16"));

	GWidgetInit disInit = { { 5, 80, 230, 40, TRUE, hKickerCont }, "Discharge", 0, 0, 0 };
	handles.hDischarge = gwinButtonCreate(0, &disInit);
	gwinSetFont(handles.hDischarge, gdispOpenFont("DejaVuSans16"));

	gtimerStart(&handles.flashTimer, &flashFunc, 0, TRUE, 150);

	return hTop;
}

void MainStatusKdUpdate(KdFeedback* pFb)
{
	static char bat[10];
	static char cur[10];
	static char cap[10];
	static char chg[10];
	static char ir[10];

	snprintf(bat, 10, "%5.2fV", pFb->power.bat);
	if(pFb->power.bat < 13.0f)
		gwinProgressbarSetRange(handles.hBatProg, 10500, 12600);
	else
		gwinProgressbarSetRange(handles.hBatProg, 13600, 16800);

	gwinProgressbarSetPosition(handles.hBatProg, pFb->power.bat*1000);
	gwinSetText(handles.hBatProg, bat, FALSE);

	snprintf(cur, 10, "%6.1fW", pFb->power.cur*pFb->power.bat);
	gwinSetText(handles.hCur, cur, FALSE);

	snprintf(cap, 10, "%5.1fV", pFb->kicker.cap);
	gwinProgressbarSetPosition(handles.hCapProg, (int)pFb->kicker.cap);
	gwinSetText(handles.hCapProg, cap, FALSE);

	snprintf(chg, 10, "%5.2fA", pFb->kicker.chg);
	gwinSetText(handles.hChg, chg, FALSE);

	snprintf(ir, 10, "%4hu/%4hu", pFb->barrier.level[0], pFb->barrier.level[1]);
	gwinSetText(handles.hBarrier, ir, FALSE);

	if(pFb->kicker.cap > 10.0f)
	{
		if(gtimerIsActive(&handles.flashTimer) == FALSE)
			gtimerStart(&handles.flashTimer, &flashFunc, 0, TRUE, 150);
	}
	else
	{
		gtimerStop(&handles.flashTimer);
		gwinSetVisible(handles.hCharged, FALSE);
	}
}

void MainStatusWifiUpdate(ConfigNetwork* pCfg)
{
	static char buf[10];

	BotCoverSetId(handles.hBotCover, pCfg->botId);

	snprintf(buf, 10, "%3hu", (uint16_t)pCfg->channel);
	gwinSetText(handles.hChannel, buf, FALSE);

	if(pCfg->speed == 2)
		gwinSetText(handles.hSpeed, "2MBit/s", FALSE);
	else if(pCfg->speed == 0)
		gwinSetText(handles.hSpeed, "250kBit/s", FALSE);
	else
		gwinSetText(handles.hSpeed, "1MBit/s", FALSE);
}

void MainStatusHwIdUpdate(uint8_t id)
{
	static char buf[10];

	snprintf(buf, 10, "%3hu", (uint16_t)id);

	gwinSetText(handles.hHwId, buf, FALSE);
}
