/*
 * test_dribbler.c
 *
 *  Created on: 14.12.2015
 *      Author: AndreR <andre@ryll.cc>
 */

#include "test_dribbler.h"
#include "styles.h"
#include "util/intercom.h"
#include "util/intercom_reliable.h"
#include "../test.h"
#include <stdio.h>
#include <math.h>

static struct _handles
{
	GHandle hBtnTest;

	GHandle hPwrUnitCont;
	GHandle hHallCont;
	GHandle hDynCont;

	GLabelObject* hPower;
	GLabelObject* hHall;
	GLabelObject* hDyn;

	struct
	{
		GLabelObject* hMain[3];
		GLabelObject* hMot[3];
		GLabelObject* hR[3];
		char main[3][8];
		char mot[3][8];
		char r[3][8];
	} pwrUnit;

	GLabelObject* hHallRow[6];
	char hall[6][16];

	GLabelObject* hDynamicsRow[2];
	char dyn[2][36];
} handles;

static int16_t eventHandler(GEvent* pEvent)
{
	if(pEvent->type == GEVENT_GWIN_BUTTON)
	{
		GEventGWinButton* pBtn = (GEventGWinButton*)pEvent;

		if(pBtn->gwin == handles.hBtnTest)
		{
			gwinSetVisible(handles.hPwrUnitCont, FALSE);
			gwinSetVisible(handles.hHallCont, FALSE);
			gwinSetVisible(handles.hDynCont, FALSE);

			handles.hPower->w.pstyle = &BlackWidgetStyle;
			gwinSetText(&handles.hPower->w.g, "-", FALSE);
			gwinSetVisible(&handles.hPower->w.g, TRUE);

			handles.hHall->w.pstyle = &BlackWidgetStyle;
			gwinSetText(&handles.hHall->w.g, "-", FALSE);
			gwinSetVisible(&handles.hHall->w.g, TRUE);

			handles.hDyn->w.pstyle = &BlackWidgetStyle;
			gwinSetText(&handles.hDyn->w.g, "-", FALSE);
			gwinSetVisible(&handles.hDyn->w.g, TRUE);

			TestScheduleMotorTest(IC_TEST_MOT_POWER, 4);
			TestScheduleMotorTest(IC_TEST_MOT_HALL, 4);
			TestScheduleMotorTest(IC_TEST_MOT_DYN, 4);

			gwinSetEnabled(handles.hBtnTest, FALSE);
		}
	}

	return -1;
}

void TestDribblerProgress(TestProgress* pProgress)
{
	static char buf[8];

	snprintf(buf, 8, "%3hu%%", (uint16_t)pProgress->percentComplete);

	switch(pProgress->testId)
	{
		case IC_TEST_MOT_POWER:
		{
			gwinSetText(&handles.hPower->w.g, buf, FALSE);
		}
		break;
		case IC_TEST_MOT_HALL:
		{
			gwinSetText(&handles.hHall->w.g, buf, FALSE);
		}
		break;
		case IC_TEST_MOT_DYN:
		{
			gwinSetText(&handles.hDyn->w.g, buf, FALSE);
		}
		break;
	}
}

void TestDribblerPowerUnitResult(TestMotorPowerUnitResult* pResult)
{
	for(uint8_t i = 0; i < 3; i++)
	{
		snprintf(handles.pwrUnit.main[i], 8, "%.3f", pResult->mainCur[i]);
		snprintf(handles.pwrUnit.mot[i], 8, "%.3f", pResult->motCur[i]);
		snprintf(handles.pwrUnit.r[i], 8, "%.3f", pResult->R[i]);

		gwinSetText(&handles.pwrUnit.hMain[i]->w.g, handles.pwrUnit.main[i], FALSE);
		gwinSetText(&handles.pwrUnit.hMot[i]->w.g, handles.pwrUnit.mot[i], FALSE);
		gwinSetText(&handles.pwrUnit.hR[i]->w.g, handles.pwrUnit.r[i], FALSE);

		handles.pwrUnit.hR[i]->w.pstyle = &BlackWidgetStyle;
	}

	uint8_t warn = 0;
	uint8_t fail = 0;

	for(uint8_t i = 0; i < 3; i++)
	{
		if(pResult->R[i] < 0.15f || pResult->R[i] > 0.3f)
		{
			warn = 1;
			handles.pwrUnit.hR[i]->w.pstyle = &YellowTextStyle;
		}

		if(pResult->R[i] < 0.1f || pResult->R[i] > 0.5f)
		{
			fail = 1;
			handles.pwrUnit.hR[i]->w.pstyle = &RedTextStyle;
		}
	}

	if(fail)
	{
		gwinSetText(&handles.hPower->w.g, "FAIL", FALSE);
		handles.hPower->w.pstyle = &RedTextStyle;
	}
	else if(warn)
	{
		gwinSetText(&handles.hPower->w.g, "WARN", FALSE);
		handles.hPower->w.pstyle = &YellowTextStyle;
	}
	else
	{
		gwinSetText(&handles.hPower->w.g, "OK", FALSE);
		handles.hPower->w.pstyle = &GreenTextStyle;
	}

	gwinSetVisible(&handles.hPower->w.g, FALSE);
	gwinSetVisible(handles.hPwrUnitCont, TRUE);
}

void TestDribblerHallResult(TestMotorHallResult* pResult)
{
	uint8_t warn = 0;
	uint8_t fail = 0;

	for(uint8_t i = 0; i < 6; i++)
	{
		snprintf(handles.hall[i], 16, "%3hd - %3hd - %3hd",
				pResult->pos[i].fwd, pResult->pos[i].ref, pResult->pos[i].rev);

		gwinSetText(&handles.hHallRow[i]->w.g, handles.hall[i], FALSE);
		handles.hHallRow[i]->w.pstyle = &BlackWidgetStyle;

		int16_t diffFwd = pResult->pos[i].ref - pResult->pos[i].fwd;
		if(i == 0)
			diffFwd += 360;

		int16_t diffRev = pResult->pos[i].rev - pResult->pos[i].ref;

		if(diffFwd > 35 || diffRev > 35)
		{
			warn = 1;
			handles.hHallRow[i]->w.pstyle = &YellowTextStyle;
		}

		if(diffFwd > 40 || diffRev > 40)
		{
			fail = 1;
			handles.hHallRow[i]->w.pstyle = &RedTextStyle;
		}
	}

	if(fail)
	{
		gwinSetText(&handles.hHall->w.g, "FAIL", FALSE);
		handles.hHall->w.pstyle = &RedTextStyle;
	}
	else if(warn)
	{
		gwinSetText(&handles.hHall->w.g, "WARN", FALSE);
		handles.hHall->w.pstyle = &YellowTextStyle;
	}
	else
	{
		gwinSetText(&handles.hHall->w.g, "OK", FALSE);
		handles.hHall->w.pstyle = &GreenTextStyle;
	}

	gwinSetVisible(&handles.hHall->w.g, FALSE);
	gwinSetVisible(handles.hHallCont, TRUE);
}

void TestDribblerDynamicResult(TestMotorDynamicsResult* pResult)
{
	uint8_t warn = 0;
	uint8_t fail = 0;

	snprintf(handles.dyn[0], 36, "R: %.3f (0.2), k: %.2f (2.54)", pResult->R, pResult->k*1000.0f);
	snprintf(handles.dyn[1], 36, "I: %.3f (1.2), v: %.1f (3150)", pResult->cur, pResult->speed);

	gwinSetText(&handles.hDynamicsRow[0]->w.g, handles.dyn[0], FALSE);
	gwinSetText(&handles.hDynamicsRow[1]->w.g, handles.dyn[1], FALSE);

	handles.hDynamicsRow[0]->w.pstyle = &BlackWidgetStyle;
	handles.hDynamicsRow[1]->w.pstyle = &BlackWidgetStyle;

	if(pResult->R < 0.35f || pResult->R > 0.55f)
	{
		warn = 1;
		handles.hDynamicsRow[0]->w.pstyle = &YellowTextStyle;
	}

	if(pResult->R < 0.25f || pResult->R > 0.9f)
	{
		fail = 1;
		handles.hDynamicsRow[0]->w.pstyle = &RedTextStyle;
	}

	if(pResult->k < 0.0022f || pResult->k > 0.0027f)
	{
		fail = 1;
		handles.hDynamicsRow[0]->w.pstyle = &RedTextStyle;
	}

	if(pResult->cur < 0.9f || pResult->cur > 1.8f)
	{
		warn = 1;
		handles.hDynamicsRow[1]->w.pstyle = &YellowTextStyle;
	}

	if(pResult->cur < 0.6f || pResult->cur > 2.8f)
	{
		fail = 1;
		handles.hDynamicsRow[1]->w.pstyle = &RedTextStyle;
	}

	if(pResult->speed < 2800 || pResult->speed > 3200)
	{
		warn = 1;
		handles.hDynamicsRow[1]->w.pstyle = &YellowTextStyle;
	}

	if(pResult->speed < 2500 || pResult->speed > 3500)
	{
		fail = 1;
		handles.hDynamicsRow[1]->w.pstyle = &RedTextStyle;
	}

	if(fail)
	{
		gwinSetText(&handles.hDyn->w.g, "FAIL", FALSE);
		handles.hDyn->w.pstyle = &RedTextStyle;
	}
	else if(warn)
	{
		gwinSetText(&handles.hDyn->w.g, "WARN", FALSE);
		handles.hDyn->w.pstyle = &YellowTextStyle;
	}
	else
	{
		gwinSetText(&handles.hDyn->w.g, "OK", FALSE);
		handles.hDyn->w.pstyle = &GreenTextStyle;
	}

	gwinSetEnabled(handles.hBtnTest, TRUE);

	gwinSetVisible(&handles.hDyn->w.g, FALSE);
	gwinSetVisible(handles.hDynCont, TRUE);
}

GHandle TestDribblerCreate()
{
    GWidgetInit wi = { { 0, 60, 240, 260, FALSE, 0 }, "Dribbler", 0, &eventHandler, 0 };
    GHandle hTop = gwinContainerCreate(0, &wi, 0);

	// Power Unit
	GWidgetInit powerUnitInit = { { 5, 5, 110, 20, TRUE, hTop }, "-- Power Unit --", 0, 0, 0 };
	gwinLabelCreate(0, &powerUnitInit);

	GWidgetInit powerUnitContInit = { { 5, 25, 110, 120, FALSE, hTop }, 0, 0, 0, 0 };
	handles.hPwrUnitCont = gwinContainerCreate(0, &powerUnitContInit, 0);

	GWidgetInit powerUnitMainInit = { { 0, 0, 60, 20, TRUE, handles.hPwrUnitCont }, "Main", 0, 0, 0 };
	gwinLabelCreate(0, &powerUnitMainInit);

	GWidgetInit powerUnitMotInit = { { 60, 0, 60, 20, TRUE, handles.hPwrUnitCont }, "Mot", 0, 0, 0 };
	gwinLabelCreate(0, &powerUnitMotInit);

	GWidgetInit powerUnitR1Init = { { 0, 80, 30, 20, TRUE, handles.hPwrUnitCont }, "  R1", 0, 0, 0 };
	gwinLabelCreate(0, &powerUnitR1Init);

	GWidgetInit powerUnitR2Init = { { 38, 80, 30, 20, TRUE, handles.hPwrUnitCont }, "  R2", 0, 0, 0 };
	gwinLabelCreate(0, &powerUnitR2Init);

	GWidgetInit powerUnitR3Init = { { 76, 80, 30, 20, TRUE, handles.hPwrUnitCont }, "  R3", 0, 0, 0 };
	gwinLabelCreate(0, &powerUnitR3Init);

	for(uint8_t j = 0; j < 3; j++)
	{
		GWidgetInit powerUnitMain1Init = { { 0, 20+j*20, 60, 20, TRUE, handles.hPwrUnitCont }, "-", 0, 0, 0 };
		handles.pwrUnit.hMain[j] = (GLabelObject*)gwinLabelCreate(0, &powerUnitMain1Init);

		GWidgetInit powerUnitMot1Init = { { 60, 20+j*20, 60, 20, TRUE, handles.hPwrUnitCont }, "-", 0, 0, 0 };
		handles.pwrUnit.hMot[j] = (GLabelObject*)gwinLabelCreate(0, &powerUnitMot1Init);

		GWidgetInit powerUnitRInit = { { j*38, 100, 60, 20, TRUE, handles.hPwrUnitCont }, "-", 0, 0, 0 };
		handles.pwrUnit.hR[j] = (GLabelObject*)gwinLabelCreate(0, &powerUnitRInit);
	}

	// Hall Sensors
	GWidgetInit hallInit = { { 125, 5, 110, 20, TRUE, hTop }, "- Hall Sensors -", 0, 0, 0 };
	gwinLabelCreate(0, &hallInit);

	GWidgetInit hallContInit = { { 125, 25, 110, 160, FALSE, hTop }, 0, 0, 0, 0 };
	handles.hHallCont = gwinContainerCreate(0, &hallContInit, 0);

	for(uint8_t j = 0; j < 6; j++)
	{
		GWidgetInit hallRowInit = { { 0, j*20, 110, 20, TRUE, handles.hHallCont }, "  x -   x -   x", 0, 0, 0 };
		handles.hHallRow[j] = (GLabelObject*)gwinLabelCreate(0, &hallRowInit);
	}

	// Dynamics
	GWidgetInit dynInit = { { 5, 140, 230, 20, TRUE, hTop }, "-- Dynamics --", 0, 0, 0 };
	gwinLabelCreate(0, &dynInit);

	GWidgetInit dynContInit = { { 5, 160, 230, 60, FALSE, hTop }, 0, 0, 0, 0 };
	handles.hDynCont = gwinContainerCreate(0, &dynContInit, 0);

	GWidgetInit dynRow1Init = { { 0, 0, 230, 20, TRUE, handles.hDynCont }, "-", 0, 0, 0 };
	handles.hDynamicsRow[0] = (GLabelObject*)gwinLabelCreate(0, &dynRow1Init);

	GWidgetInit dynRow2Init = { { 0, 20, 230, 20, TRUE, handles.hDynCont }, "-", 0, 0, 0 };
	handles.hDynamicsRow[1] = (GLabelObject*)gwinLabelCreate(0, &dynRow2Init);

    // Container for progress
	GWidgetInit powerUnitProgInit = { { 45, 25, 110, 20, FALSE, hTop }, " -", 0, 0, 0 };
	handles.hPower = (GLabelObject*)gwinLabelCreate(0, &powerUnitProgInit);

	GWidgetInit hallProgInit = { { 165, 25, 110, 20, FALSE, hTop }, " -", 0, 0, 0 };
	handles.hHall = (GLabelObject*)gwinLabelCreate(0, &hallProgInit);

	GWidgetInit dynProgInit = { { 45, 160, 230, 20, FALSE, hTop }, " -", 0, 0, 0 };
	handles.hDyn = (GLabelObject*)gwinLabelCreate(0, &dynProgInit);

	GWidgetInit testBtnInit = { { 125, 225, 110, 30, TRUE, hTop }, "Test", 0, 0, 0 };
	handles.hBtnTest = gwinButtonCreate(0, &testBtnInit);

    return hTop;
}
