/*
 * cli.c
 *
 *  Created on: 19.11.2012
 *      Author: AndreR
 */

#include "cli.h"

#include "arm_math.h"
#include "led.h"
#include "util/console.h"
#include "util/network_print.h"
#include "util/sys_time.h"
#include "util/flash.h"
#include "util/flash_fs.h"
#include "util/log.h"
#include "util/arm_mat_util_f32.h"
#include "util/intercom.h"
#include "intercom_constants.h"
#include "spi1.h"
#include "drive.h"
#include "cpu_load.h"
#include "motors.h"
#include "test.h"

CLI cli;

void processCommand();

void CLIInit()
{
	chMtxObjectInit(&cli.procBufMutex);

	static uint8_t rxData[CONSOLE_RX_SIZE];
	static uint8_t txData[CONSOLE_TX_SIZE];

	ConsoleData conData;
	conData.pDataTx = txData;
	conData.txSize = CONSOLE_TX_SIZE;
	conData.pDataRx = rxData;
	conData.rxSize = CONSOLE_RX_SIZE;
	conData.printSize = CONSOLE_PRINT_SIZE;
	conData.txCallback = 0;
	conData.pUserData = 0;

	ConsoleInit(conData);

#ifdef MOT_LEFT
	ConsoleIntercomAdapterInit(&cli.adapter, INTERCOM_ADDRESS_LEFT, INTERCOM_ADDRESS_MAIN);
#else
	ConsoleIntercomAdapterInit(&cli.adapter, INTERCOM_ADDRESS_RIGHT, INTERCOM_ADDRESS_MAIN);
#endif
}

void CLITask(void* params)
{
	(void)params;

	chRegSetThreadName("CLI");

#ifdef ENV_BOOT
#ifdef MOT_LEFT
	ConsolePrint("--- Bootloader (Left) ---\r\n");
#else
	ConsolePrint("--- Bootloader (Right) ---\r\n");
#endif
#else
#ifdef MOT_LEFT
	ConsolePrint("--- Left ---\r\n");
#else
	ConsolePrint("--- Right ---\r\n");
#endif
#endif

	while(1)
	{
		chThdSleepMilliseconds(20);

		if(ConsoleIsCmdReady())
		{
			processCommand();

			ConsoleCmdProcessed();
		}

	}
}

void processCommand()
{
	uint16_t u16;

	if(ConsoleCmpCmd("test"))
	{
		ConsolePrint("Number: %d\r\n", 43);
	}

	if(ConsoleCmpCmd("test2"))
	{
		ConsolePrint("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789\r\n");
	}

	if(ConsoleCmpCmd("cpu"))
	{
		CPULoadPrintUsage();
	}

	if(ConsoleCmpCmd("tasks"))
	{
		CPULoadPrintTasks();
	}

	if(ConsoleCmpCmd("timsr"))
	{
		ConsolePrint("Resetting SR register\r\n");

		TIM1->SR = 0;
		TIM8->SR = 0;
	}

	if(ConsoleCmpCmd("tim"))
	{
		ConsolePrint("TIM1:\r\n");
		ConsolePrint("CR1: 0x%hX, SR: 0x%X, BDTR: 0x%X\r\n", TIM1->CR1, TIM1->SR, TIM1->BDTR);
		ConsolePrint("CCR: %hu, %hu, %hu\r\n", TIM1->CCR1, TIM1->CCR2, TIM1->CCR3);
		ConsolePrint("TIM8:\r\n");
		ConsolePrint("CR1: 0x%hX, SR: 0x%X, BDTR: 0x%X\r\n", TIM8->CR1, TIM8->SR, TIM8->BDTR);
		ConsolePrint("CCR: %hu, %hu, %hu\r\n", TIM8->CCR1, TIM8->CCR2, TIM8->CCR3);
		ConsolePrint("outLevel: %d, %d\r\n", pFront->outLevel, pRear->outLevel);
	}

	if(ConsoleCmpCmd("stubs"))
	{
		EXTI->SWIER = (1 << 3);
	}

	if(ConsoleCmpCmd("adc"))
	{
		for(uint8_t a = 0; a < 2; a++)
		{
			ConsolePrint("ADC%hu: ", (uint16_t)a);
			for(uint8_t i = 0; i < DRIVE_ADC_SAMPLES; i++)
			{
				ConsolePrint("% 5hd ", pRear->adc[a][i]);
			}
			ConsolePrint("\r\n");
		}
	}

//	if(ConsoleCmpCmd("current"))
//	{
//		ConsolePrint("iA: %d, iB: %d, iC: %d\r\n", (int32_t)(pRear->i[0]*1000), (int32_t)(pRear->i[1]*1000), (int32_t)(pRear->i[2]*1000));
//		ConsolePrint("iAlpha: %d, iBeta: %d\r\n", (int32_t)(pRear->iAlpha*1000), (int32_t)(pRear->iBeta*1000));
//		ConsolePrint("iD: %d, iQ: %d\r\n", (int32_t)(rear.pOut->iD*1000), (int32_t)(rear.pOut->iQ*1000));
//	}

	if(ConsoleCmpCmd("enc"))
	{
		float angle = (pRear->pTIMEncoder->CNT%256)*1.40625f;
		ConsolePrint("ENC pos: %u\r\n", (uint32_t)(angle*1000));
	}

	if(ConsoleCmpCmd("speed"))
	{
		ConsolePrint("Front vel: %d\r\n", (int32_t)(pFront->curVelocity*1000));
	}

/*	if(ConsoleCmpCmd("input"))
	{
		DriveMain2Mot* pFront = &spi1.pRxData->periodic.mot[0];
		DriveMain2Mot* pRear = &spi1.pRxData->periodic.mot[1];

		ConsolePrint("Front:\r\n");
		ConsolePrint("Vel: %d\r\niD:  %d\r\niQ:  %d\r\n",
				(int32_t)(pFront->velocity*1000), (int32_t)(pFront->iD*1000), (int32_t)(pFront->iQ*1000));
		ConsolePrint("Rear:\r\n");
		ConsolePrint("Vel: %d\r\niD:  %d\r\niQ:  %d\r\n",
				(int32_t)(pRear->velocity*1000), (int32_t)(pRear->iD*1000), (int32_t)(pRear->iQ*1000));
	}

	if(ConsoleCmpCmd("output"))
	{
		ConsolePrint("Front:\r\n");
		ConsolePrint("Vel: %d\r\niD:  %d\r\niQ:  %d\r\n",
				(int32_t)(front.pOut->velocity*1000), (int32_t)(front.pOut->iD*1000), (int32_t)(front.pOut->iQ*1000));
		ConsolePrint("Rear:\r\n");
		ConsolePrint("Vel: %d\r\niD:  %d\r\niQ:  %d\r\n",
				(int32_t)(rear.pOut->velocity*1000), (int32_t)(rear.pOut->iD*1000), (int32_t)(rear.pOut->iQ*1000));
	}
*/

	uint32_t p, i, d;
	if(ConsoleScanCmd("speedpid %u %u %u", &p, &i, &d) == 3)
	{
		ConsolePrint("Setting Speed PID: %u, %u, %u\r\n", p, i, d);

		DriveSpeedSetPIDParams(&drive[0], p*0.001f, i*0.001f, d*0.000001f);
		DriveSpeedSetPIDParams(&drive[1], p*0.001f, i*0.001f, d*0.000001f);
	}

	if(ConsoleScanCmd("torquepid %u %u %u", &p, &i, &d) == 3)
	{
		ConsolePrint("Setting Torque PID: %u, %u, %u\r\n", p, i, d);

		DriveTorqueSetPIDParams(&drive[0], p*0.001f, i*0.001f, d*0.001f);
		DriveTorqueSetPIDParams(&drive[1], p*0.001f, i*0.001f, d*0.001f);
	}

	if(ConsoleCmpCmd("bat"))
	{
		ConsolePrint("Front: %hu\tRear: %hu\r\n", (uint16_t)(pFront->bat*1000), (uint16_t)(pRear->bat*1000));
		ConsolePrint("Front: %hu\tRear: %hu\r\n", (uint16_t)(pFront->cur*1000), (uint16_t)(pRear->cur*1000));
	}

	if(ConsoleCmpCmd("time"))
	{
		ConsolePrint("Time: %uus\r\n", SysTimeUSec());
	}

	if(ConsoleCmpCmd("hall"))
	{
		ConsolePrint("Front: %hu\tRear: %hu\r\n", pFront->halPos, pRear->halPos);
	}

	if(ConsoleScanCmd("ignore %hu", &u16) == 1)
	{
		if(u16)
		{
			motors.ignoreMainCmds = 1;
			ConsolePrint("Ingoring commands from main\r\n");
		}
		else
		{
			motors.ignoreMainCmds = 0;
			ConsolePrint("Main cmds re-enabled\r\n");
		}
	}

	if(ConsoleCmpCmd("ctrl"))
	{
		ConsolePrint("Ctrl: %hu\r\n", (uint16_t)pFront->ctrlMode);
	}

 	if(ConsoleCmpCmd("ctrl none"))
	{
 		DriveSetCtrlMode(&drive[0], DRIVE_CTRL_MODE_NONE);
 		DriveSetCtrlMode(&drive[1], DRIVE_CTRL_MODE_NONE);

		ConsolePrint("Controller disabled\r\n");
	}

	if(ConsoleCmpCmd("ctrl speed"))
	{
 		DriveSetCtrlMode(&drive[0], DRIVE_CTRL_MODE_PID_SPEED);
 		DriveSetCtrlMode(&drive[1], DRIVE_CTRL_MODE_PID_SPEED);

		ConsolePrint("Speed PID controller active\r\n");
	}

	if(ConsoleCmpCmd("ctrl torque"))
	{
 		DriveSetCtrlMode(&drive[0], DRIVE_CTRL_MODE_PID_TORQUE);
 		DriveSetCtrlMode(&drive[1], DRIVE_CTRL_MODE_PID_TORQUE);


		ConsolePrint("Torque PID controller active\r\n");
	}

	if(ConsoleCmpCmd("com none"))
	{
		DriveSetMode(pFront, DRIVE_MODE_DISABLE);
		DriveSetMode(pRear, DRIVE_MODE_DISABLE);

		ConsolePrint("Commutation mode: NONE\r\n");
	}

	if(ConsoleCmpCmd("com block"))
	{
		DriveSetMode(pFront, DRIVE_MODE_BLOCK);
		DriveSetMode(pRear, DRIVE_MODE_BLOCK);

		ConsolePrint("Commutation mode: BLOCK\r\n");
	}

	int16_t i16;
	if(ConsoleScanCmd("vel %hd", &i16) == 1)
	{
		ConsolePrint("Set velocity to %hdRPS\r\n", i16);

		DriveSpeedSetVelocity(&drive[0], (float)i16);
	}

	if(ConsoleScanCmd("vol %hd", &i16) == 1)
	{
		ConsolePrint("Set voltage to %hdmV\r\n", i16);

		float volt = i16*0.001f;

		DriveSetVoltage(&drive[0], volt);
	}

	if(ConsoleScanCmd("current %hd", &i16) == 1)
	{
		ConsolePrint("Set current to %hdmA\r\n", i16);

		float current = i16*0.001f;

		pFront->torque.setCurrent = current;
	}

	if(ConsoleCmpCmd("state"))
	{
		ConsolePrint("Voltage: %dmV\r\nSpeed: %dmRPS\r\n",
				(int32_t)(pFront->setVoltage*1000), (int32_t)(pFront->curVelocity*1000));
		ConsolePrint("RMS Current: %dmA\r\nest Current: %dmA\r\n",
				(int32_t)(pFront->rmsCurrent*1000), (int32_t)(pFront->estCurrent*1000));
	}

	uint16_t vref, angle;
	if(ConsoleScanCmd("va %hu %hu", &vref, &angle) == 2)
	{
//		float a = ((float)angle)*PI/180.0;
		float r = ((float)vref)/1000.0f;

		DriveSetVoltageAngle(pRear, r, angle);
	}

	if(ConsoleScanCmd("test dyn %hu", &u16) == 1)
	{
		ConsolePrint("Motor dynamics test\r\n");

		Drive* pDrive = pFront;
		if(u16)
			pDrive = pRear;

		TestMotorDynamicsResult res;
		TestMotorDynamics(pDrive, &res);

		ConsolePrint("R: %d, k: %d\r\n",
				(int32_t)(res.R*1000.0f), (int32_t)(res.k*1000000.0f));
		ConsolePrint("Current: %dmA, Speed: %drad/s\r\n",
				(int32_t)(res.cur*1000.0f), (int32_t)res.speed);
	}

	uint16_t mot;
	if(ConsoleScanCmd("test enc %hu", &mot) == 1)
	{
		ConsolePrint("Encoder test\r\n");

		Drive* pDrive = pFront;
		if(mot)
			pDrive = pRear;

		TestMotorEncoderResult res;
		TestMotorEncoder(pDrive, &res);

		ConsolePrint("Expected: %hu, Measured: %hu\r\n", res.expected, res.measured);
	}

	if(ConsoleScanCmd("test hall %hu", &mot) == 1)
	{
		Drive* pDrive = pFront;
		if(mot)
			pDrive = pRear;

		ConsolePrint("Motor hall test\r\n");

		TestMotorHallResult res;
		TestMotorHall(pDrive, &res);

		for(uint16_t i = 0; i < 6; i++)
		{
			ConsolePrint("%3hd - %3hd - %3hd\r\n", res.pos[i].fwd, res.pos[i].ref, res.pos[i].rev);
		}
	}

	if(ConsoleScanCmd("test unit %hu %hu", &u16, &vref) == 2)
	{
		ConsolePrint("Power unit test with %humV\r\n", vref);

		Drive* pDrive = pFront;
		if(u16)
			pDrive = pRear;

		TestMotorPowerUnitResult res;
		TestMotorPowerUnit(pDrive, vref*0.001f, &res);

		for(uint16_t ph = 0; ph < 3; ph++)
		{
			ConsolePrint("%hu: Main: %dmA, Mot: %dmA\r\n",
					ph, (int32_t)(res.mainCur[ph]*1000.0f), (int32_t)(res.motCur[ph]*1000.0f));
		}

		ConsolePrint("R1: %d, R2: %d, R3: %d\r\n", (int32_t)(res.R[0]*1000.0f), (int32_t)(res.R[1]*1000.0f), (int32_t)(res.R[2]*1000.0f));
	}

	if(ConsoleScanCmd("test %hu", &mot) == 1)
	{
		TestMotorPowerUnitResult power;
		TestMotorHallResult hall;
		TestMotorEncoderResult enc;
		TestMotorDynamicsResult dyn;

		Drive* pDrive = pFront;
		if(mot)
			pDrive = pRear;

		ConsolePrint("--- Power Unit Test ---\r\n");
		TestMotorPowerUnit(pDrive, 1.0f, &power);
		for(uint16_t ph = 0; ph < 3; ph++)
		{
			ConsolePrint("%hu: Main: %dmA, Mot: %dmA\r\n",
					ph, (int32_t)(power.mainCur[ph]*1000.0f), (int32_t)(power.motCur[ph]*1000.0f));
		}
		ConsolePrint("R1: %d, R2: %d, R3: %d (expected: 600)\r\n", (int32_t)(power.R[0]*1000.0f), (int32_t)(power.R[1]*1000.0f), (int32_t)(power.R[2]*1000.0f));

		ConsolePrint("--- Motor Hall Test ---\r\n");
		TestMotorHall(pDrive, &hall);
		for(uint16_t i = 0; i < 6; i++)
		{
			ConsolePrint("%3hd - %3hd - %3hd\r\n", hall.pos[i].fwd, hall.pos[i].ref, hall.pos[i].rev);
		}

		ConsolePrint("--- Encoder Test ---\r\n");
		TestMotorEncoder(pDrive, &enc);
		ConsolePrint("Measured: %hu (%hu)\r\n", enc.measured, enc.expected);

		ConsolePrint("--- Motor Dynamics Test ---\r\n");
		TestMotorDynamics(pDrive, &dyn);
		ConsolePrint("R: %d (1200), k: %d (25500)\r\n",
				(int32_t)(dyn.R*1000.0f), (int32_t)(dyn.k*1000000.0f));
		ConsolePrint("Current: %dmA (~200), Speed: %drad/s (~320)\r\n",
				(int32_t)(dyn.cur*1000.0f), (int32_t)dyn.speed);
	}

	if(ConsoleCmpCmd("full test"))
	{
		TestMotorPowerUnitResult power;
		TestMotorHallResult hall;
		TestMotorEncoderResult enc;
		TestMotorDynamicsResult dyn;

		for(uint8_t i = 0; i < 2; i++)
		{
			if(i == 0)
				ConsolePrint("\r\n=== FRONT ===\r\n");
			else
				ConsolePrint("\r\n=== REAR ===\r\n");

			ConsolePrint("--- Power Unit Test ---\r\n");
			TestMotorPowerUnit(&drive[i], 1.0f, &power);
			for(uint16_t ph = 0; ph < 3; ph++)
			{
				ConsolePrint("%hu: Main: %dmA, Mot: %dmA\r\n",
						ph, (int32_t)(power.mainCur[ph]*1000.0f), (int32_t)(power.motCur[ph]*1000.0f));
			}
			ConsolePrint("R1: %d, R2: %d, R3: %d (expected: 600)\r\n", (int32_t)(power.R[0]*1000.0f), (int32_t)(power.R[1]*1000.0f), (int32_t)(power.R[2]*1000.0f));

			ConsolePrint("--- Motor Hall Test ---\r\n");
			TestMotorHall(&drive[i], &hall);
			for(uint16_t i = 0; i < 6; i++)
			{
				ConsolePrint("%3hd - %3hd - %3hd\r\n", hall.pos[i].fwd, hall.pos[i].ref, hall.pos[i].rev);
			}

			ConsolePrint("--- Encoder Test ---\r\n");
			TestMotorEncoder(&drive[i], &enc);
			ConsolePrint("Measured: %hu (%hu)\r\n", enc.measured, enc.expected);

			ConsolePrint("--- Motor Dynamics Test ---\r\n");
			TestMotorDynamics(&drive[i], &dyn);
			ConsolePrint("R: %d (1200), k: %d (25500)\r\n",
					(int32_t)(dyn.R*1000.0f), (int32_t)(dyn.k*1000000.0f));
			ConsolePrint("Current: %dmA (~200), Speed: %drad/s (~320)\r\n",
					(int32_t)(dyn.cur*1000.0f), (int32_t)dyn.speed);
		}
	}

	if(ConsoleCmpCmd("flash hex"))
	{
		for(uint32_t i = 0; i < flashFS.bytesDel+flashFS.bytesUsed+48; i += 8)
		{
			uint8_t* d = (uint8_t*)(flashFS.flashAddr+i);
			ConsolePrint("0x%04X: %02X %02X %02X %02X %02X %02X %02X %02X\r\n",
					i, d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7]);
		}
	}

	if(ConsoleCmpCmd("flash list"))
	{
		FlashFSListFiles();
	}

	if(ConsoleCmpCmd("flash erase"))
	{
		ConsolePrint("Erasing flash\r\n");

		FlashErase(flashFS.flashAddr, flashFS.flashAddr+flashFS.flashSize);
	}

	if(ConsoleScanCmd("log %hu", &u16) == 1)
	{
		LogPrintAllRecent(u16);
	}

	if(ConsoleCmpCmd("netprint"))
	{
#ifdef MOT_LEFT
		NetworkPrint("Hello Network - LEFT\n");
#else
		NetworkPrint("Hello Network - RIGHT\n");
#endif
	}
}
