/*
 * motors.c
 *
 *  Created on: 10.06.2014
 *      Author: AndreR
 */

#include "motors.h"

#include <main/ctrl.h>
#include "util/intercom.h"
#include "intercom_constants.h"
#include "drive.h"

Motors motors;

void MotorsRegularIntercom()
{
	IntercomHeader iHeader;
	iHeader.address = INTERCOM_ADDRESS_MAIN;
	iHeader.section = INTERCOM_SECTION_MOTORS;
	iHeader.command = IC_MOTORS_MOT2MAIN;

	MotorsMot2Main mot2Main;
#ifdef MOT_RIGHT
	mot2Main.source = MOT_SOURCE_RIGHT;
#else
	mot2Main.source = MOT_SOURCE_LEFT;
#endif
	for(uint8_t m = 0; m < 2; m++)
	{
		mot2Main.mot[m].velocity = drive[m].curVelocity;
		mot2Main.mot[m].rmsCurrent = drive[m].rmsCurrent;
		// TODO: fix these estimates
		mot2Main.mot[m].torque = 0;
		mot2Main.mot[m].torqueTimestamp = 0;
//		mot2Main.mot[m].torque = drive[m].estTorque;
//		mot2Main.mot[m].torqueTimestamp = drive[m].estTimestamp;
		mot2Main.mot[m].velocityTimestamp = drive[m].curVelocityTimestamp;
		mot2Main.mot[m].rmsCurrentTimestamp = drive[m].rmsCurrentTimestamp;
		mot2Main.mot[m].setVoltage = drive[m].setVoltage;
		mot2Main.mot[m].setVoltageTimestamp = drive[m].setVoltageTimestamp;
	}

	IntercomSend(&iHeader, sizeof(mot2Main), (uint8_t*)&mot2Main);
}

static void motorsCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;

	if(pHeader->command == IC_MOTORS_MAIN2MOT && dataLength >= sizeof(MotorsMain2Mot))
	{
		motors.lastCmdRx = chVTGetSystemTime();

		MotorsMain2Mot* pMain2Mot = (MotorsMain2Mot*)pData;

		for(uint8_t m = 0; m < 2; m++)
		{
			Drive* pDrive = &drive[m];

			pDrive->bat = pMain2Mot->bat;
			pDrive->cur = pMain2Mot->cur;

			if(motors.ignoreMainCmds)
				continue;

			if(pMain2Mot->ctrlMode == MOT_CTRL_OFF)
			{
				DriveSetCtrlMode(&drive[m], DRIVE_CTRL_MODE_NONE);
				DriveSetMode(&drive[m], DRIVE_MODE_DISABLE);
				DriveSpeedSetVelocity(&drive[m], 0);
			}
			else
			{
				DriveSetMode(&drive[m], DRIVE_MODE_BLOCK);
			}

			switch(pMain2Mot->ctrlMode)
			{
				case MOT_CTRL_VELOCITY:
				{
					DriveSetCtrlMode(&drive[m], DRIVE_CTRL_MODE_PID_SPEED);
					DriveSpeedSetVelocity(&drive[m], pMain2Mot->mot[m].velocity);
				}
				break;
				case MOT_CTRL_VOLTAGE:
				{
					DriveSetCtrlMode(&drive[m], DRIVE_CTRL_MODE_NONE);
					DriveSetVoltage(&drive[m], pMain2Mot->mot[m].voltage);
				}
				break;
				case MOT_CTRL_TORQUE:
				{
					DriveSetCtrlMode(&drive[m], DRIVE_CTRL_MODE_PID_TORQUE);
					DriveTorqueSetTorque(&drive[m], pMain2Mot->mot[m].torque);
				}
				break;
				default: break;
			}
		}
	}
}

void MotorsInit()
{
	motors.ignoreMainCmds = 0;

	IntercomClient client;
	client.pUserData = 0;
	client.rxCallback = &motorsCallback;
	client.section = INTERCOM_SECTION_MOTORS;

	IntercomAddClient(client);

	motors.lastCmdRx = chVTGetSystemTime();
}
