/*
 * log_msgs.c
 *
 *  Created on: 29.02.2016
 *      Author: AndreR
 */

#include "log_msgs.h"
#include "struct_ids.h"

const LogMessageDesc logMessageDescriptions[9] = {
	{ SID_GYRO_RAW, "gyro_raw", 3, (ElementDesc[]) {
		{ INT16, "X", "raw", "X axis" },
		{ INT16, "Y", "raw", "Y axis" },
		{ INT16, "Z", "raw", "Z axis" },
	}, },
	{ SID_ACC_RAW, "acc_raw", 3, (ElementDesc[]) {
		{ INT16, "X", "raw", "X axis" },
		{ INT16, "Y", "raw", "Y axis" },
		{ INT16, "Z", "raw", "Z axis" },
	}, },
	{ SID_CTRL_INPUT, "ctrl_input", 30, (ElementDesc[]) {
		{ UINT32,	"vel_t", "us",	"Motor velocity timestamp" },
		{ FLOAT,	"vel_m1", "rad/s","Motor 1 angular velocity" },
		{ FLOAT,	"vel_m2", "rad/s","Motor 2 angular velocity" },
		{ FLOAT,	"vel_m3", "rad/s","Motor 3 angular velocity" },
		{ FLOAT,	"vel_m4", "rad/s","Motor 4 angular velocity" },

		{ UINT8,	"gyr_updated", "bool", "Gyroscope data updated" },
		{ UINT32,	"gyr_t", "us", "Timestamp" },
		{ FLOAT,	"gyr_x", "rad/s", "Angular velocity around X axis" },
		{ FLOAT,	"gyr_y", "rad/s", "Angular velocity around Y axis" },
		{ FLOAT,	"gyr_z", "rad/s", "Angular velocity around Z axis" },

		{ UINT8,	"acc_updated", "bool", "Accelerometer data updated" },
		{ UINT32,	"acc_t", "us", "Timestamp" },
		{ FLOAT,	"acc_x", "m/s^2", "Acceleration along X axis" },
		{ FLOAT,	"acc_y", "m/s^2", "Acceleration along Y axis" },
		{ FLOAT,	"acc_z", "m/s^2", "Acceleration along Z axis" },

		{ UINT8,	"pos_updated", "bool", "Position data updated" },
		{ UINT32,	"pos_t", "us", "Timestamp" },
		{ UINT32,	"pos_delay", "us", "Vision Delay" },
		{ FLOAT,	"pos_x", "m", "X Position" },
		{ FLOAT,	"pos_y", "m", "Y Position" },
		{ FLOAT,	"pos_z", "m", "Z Position" },
		{ UINT8,	"cam_id", "-", "Camera ID" },

		{ FLOAT,	"supply_bat", "V", "Battery voltage" },
		{ FLOAT,	"supply_cur", "A", "Current consumption" },
		{ UINT8,	"bat_empty", "bool", "Battery empty" },

		{ UINT32,	"vol_t", "us",	"Motor voltage timestamp" },
		{ FLOAT,	"vol_m1", "V","Motor 1 voltage" },
		{ FLOAT,	"vol_m2", "V","Motor 2 voltage" },
		{ FLOAT,	"vol_m3", "V","Motor 3 voltage" },
		{ FLOAT,	"vol_m4", "V","Motor 4 voltage" },
	}, },
	{ SID_CTRL_STATE, "ctrl_state", 12, (ElementDesc[]) {
		{ FLOAT,	"pos_x", "m", "X Position" },
		{ FLOAT,	"pos_y", "m", "Y Position" },
		{ FLOAT,	"pos_z", "rad", "Z Position" },

		{ FLOAT,	"vel_x", "m/s", "X Velocity" },
		{ FLOAT,	"vel_y", "m/s", "Y Velocity" },
		{ FLOAT,	"vel_z", "rad/s", "Z Velocity" },

		{ FLOAT,	"acc_x", "m/s^2", "X Acceleration" },
		{ FLOAT,	"acc_y", "m/s^2", "Y Acceleration" },
		{ FLOAT,	"acc_z", "rad/s^2", "Z Acceleration" },

		{ UINT8,	"pos_updated", "bool", "Position estimation updated" },
		{ UINT8,	"vel_updated", "bool", "Velocity estimation updated" },
		{ UINT8,	"acc_updated", "bool", "Acceleration estimation updated" },
	}, },
	{ SID_CTRL_OUTPUT, "ctrl_output", 9, (ElementDesc[]) {
		{ FLOAT,	"vel_m1", "rad/s", "Motor 1 velocity" },
		{ FLOAT,	"vel_m2", "rad/s", "Motor 2 velocity" },
		{ FLOAT,	"vel_m3", "rad/s", "Motor 3 velocity" },
		{ FLOAT,	"vel_m4", "rad/s", "Motor 4 velocity" },
		{ FLOAT,	"vol_m1", "V", "Motor 1 voltage" },
		{ FLOAT,	"vol_m2", "V", "Motor 2 voltage" },
		{ FLOAT,	"vol_m3", "V", "Motor 3 voltage" },
		{ FLOAT,	"vol_m4", "V", "Motor 4 voltage" },
		{ UINT8,	"ctrlMode", "-", "Motor controller mode"},
	}, },
	{ SID_CTRL_TIGGA_DEBUG, "ctrl_tigga_debug", 32, (ElementDesc[]) {
		{ FLOAT,	"sigma_pos_x", "m", "UKF Position Uncertainty X" },
		{ FLOAT,	"sigma_pos_y", "m", "UKF Position Uncertainty Y" },
		{ FLOAT,	"sigma_pos_z", "rad", "UKF Orientation Uncertainty" },

		{ FLOAT,	"sigma_vel_x", "m/s", "UKF Velocity Uncertainty X" },
		{ FLOAT,	"sigma_vel_y", "m/s", "UKF Velocity Uncertainty Y" },
		{ FLOAT,	"sigma_vel_z", "rad/s", "UKF Angular Velocity Uncertainty" },

		{ FLOAT,	"sigma_acc_x", "m/s^2", "UKF Acceleration Uncertainty X" },
		{ FLOAT,	"sigma_acc_y", "m/s^2", "UKF Acceleration Uncertainty Y" },
		{ FLOAT,	"sigma_acc_z", "rad/s^2", "UKF Angular Acceleration Uncertainty" },

		{ UINT8,	"ctrl_mode", "-", "0 localVel, 1 globalPos, 2 wheelVel"},

		{ FLOAT,	"set_pos_x", "m", "Setpoint Position X" },
		{ FLOAT,	"set_pos_y", "m", "Setpoint Position Y" },
		{ FLOAT,	"set_pos_z", "m", "Setpoint Orientation" },

		{ FLOAT,	"set_vel_x", "m/s", "Setpoint Velocity X" },
		{ FLOAT,	"set_vel_y", "m/s", "Setpoint Velocity Y" },
		{ FLOAT,	"set_vel_z", "m/s", "Setpoint Angular Velocity" },

		{ FLOAT,	"traj_pos_x", "m", "Global Position from Trajectory X" },
		{ FLOAT,	"traj_pos_y", "m", "Global Position from Trajectory Y"},
		{ FLOAT,	"traj_pos_z", "m", "Global Position from Trajectory Z" },

		{ FLOAT,	"traj_vel_local_x", "m/s", "Local Velocity from Trajectory X" },
		{ FLOAT,	"traj_vel_local_y", "m/s", "Local Velocity from Trajectory Y"},
		{ FLOAT,	"traj_vel_local_z", "m/s", "Local Velocity from Trajectory Z" },

		{ FLOAT,	"traj_acc_local_x", "m/s^2", "Local Acceleration from Trajectory X" },
		{ FLOAT,	"traj_acc_local_y", "m/s^2", "Local Acceleration from Trajectory Y" },
		{ FLOAT,	"traj_acc_local_z", "m/s^2", "Local Acceleration from Trajectory Z" },

		{ FLOAT,	"traj_jerk_local_x", "m/s^3", "Local Jerk from Trajectory X" },
		{ FLOAT,	"traj_jerk_local_y", "m/s^3", "Local Jerk from Trajectory Y" },
		{ FLOAT,	"traj_jerk_local_z", "m/s^3", "Local Jerk from Trajectory Z" },

		{ UINT32,	"fusion_time", "us", "Sensor fusion processing time" },
		{ UINT32,	"ctrl_time", "us", "Control processing time" },

		{ FLOAT,	"enc2real_x", "-", "Local conv. encoder to real X" },
		{ FLOAT,	"enc2real_y", "-", "Local conv. encoder to real Y" },
	}, },
	{ SID_KICKER_DRIBBLER, "kicker_dribbler", 9, (ElementDesc[]) {
		{ FLOAT,	"kicker_level", "V", "Kicker Level" },
		{ FLOAT,	"kicker_chg", "A", "Kicker Charge Current" },
		{ UINT8,	"kicker_cnt", "-", "Kick Counter" },

		{ INT16,	"dribbler_speed", "RPM", "Dribbler Speed" },
		{ FLOAT,	"dribbler_temp", "C", "Dribbler Temperature" },
		{ UINT8,	"dribbler_overheated", "bool", "Dribbler Overheated" },

		{ UINT16,	"ir_off", "mV", "IR Level - Transmitter Off" },
		{ UINT16,	"ir_on", "mV", "IR Level - Transmitter On"},
		{ UINT8,	"ir_interrupted", "bool", "Barrier interrupted" },
	}, },
	{ SID_CTRL_DRIVE_TRAIN, "cfg_drive_train", 7, (ElementDesc[]) {
		{ FLOAT,	"motor2wheel", "-", "Motor to Wheel Ratio" },
		{ FLOAT,	"motor_km", "Nm/A", "Motor Torque Constant" },
		{ FLOAT,	"motor_kn", "rad/(s*V)", "Motor Speed Constant" },
		{ FLOAT,	"motor_ke", "V/rad", "Motor back-EMF constant" },
		{ FLOAT,	"motor_kf", "Nm*s", "Motor Friction Constant" },
		{ FLOAT,	"motor_r", "Ohm", "Motor Resistance" },
		{ FLOAT,	"motor_l", "H", "Motor Inductance" },
	}, },
	{ SID_OPTICAL_RAW, "optical_raw", 6, (ElementDesc[]) {
		{ INT16,	"l_delta_x", "-", "Left Delta X" },
		{ INT16,	"l_delta_y", "-", "Left Delta Y" },
		{ UINT8,	"l_squal", "-", "Left Quality" },
		{ INT16,	"r_delta_x", "-", "Right Delta X" },
		{ INT16,	"r_delta_y", "-", "Right Delta Y" },
		{ UINT8,	"r_squal", "-", "Right Quality" },
	}, },
};
