/*

 * ctrl_tigga.h
 *
 *  Created on: 26.02.2015
 *      Author: AndreR
 */

// TIGGA is pretty clever, he can control a robot :)

#ifndef CTRL_TIGGA_H_
#define CTRL_TIGGA_H_

#include <arm_math.h>
#include <main/ctrl.h>
#include <stdint.h>
#include "util/config.h"
#include "util/ema_filter.h"
#include "util/kf.h"
#include "util/trajectory.h"
#include "util/traj_generator.h"
#include "util/data_storage.h"
#include "util/pid.h"
#include "util/ukf.h"
#include "util/lag_element.h"
#include "util/srukf.h"

typedef struct PACKED _CtrlTiggaConfigFilter
{
	struct PACKED _state
	{
		float posXY;
		float posW;
		float velXY;
		float velW;
		float accXY;
	} state;

	struct PACKED _sensor
	{
		float visXY;
		float visW;
		float gyrW;
		float accXY;
	} sensor;
} CtrlTiggaConfigFilter;

typedef struct PACKED _CtrlTiggaConfigMisc
{
	uint16_t visionDelay;
	uint16_t bsDelayMax;

	uint16_t ASR;

	float encoderAlpha;

	float dampingXY;
	float dampingW;
	float tShiftXY;
	float tShiftW;

	uint8_t useFeedforward;
} CtrlTiggaConfigMisc;

typedef struct PACKED _CtrlTiggaConfigPID
{
	struct PACKED
	{
		float kp;
		float ki;
		float kd;
	} posXY;

	struct PACKED
	{
		float kp;
		float ki;
		float kd;
	} posW;

	struct PACKED
	{
		float kp;
		float ki;
		float kd;
	} velXY;

	struct PACKED
	{
		float kp;
		float ki;
		float kd;
	} velW;
} CtrlTiggaConfigPID;

typedef struct _CtrlTigga
{
	// state estimation stuff
	SRUKF ukf;
	float ukfData[SRUKF_DATA_SIZE(8, 6, 3, 8)];
	float obsNoise[6];
	float stateNoise[8];

	// control stuff
	uint8_t mode;
	uint8_t lastMode;

	TrajGenerator trajGen;

	Trajectory2D trajXY;
	Trajectory1D trajW;

	float setpointVel[3];
	float setpointPos[3];
	float setpointWheel[4];

	PID posPIDXY;
	PID posPIDW;

	PID velPIDXY;
	PID velPIDW;

	EMAFilter encFilter[2];	// X, Y

	struct _lastVision
	{
		uint8_t camId;
		uint32_t time;	// delay subtracted
		float pos[3];
	} lastVision;

	float encPosition[3];

	float lastLocalVelOut[3];

	CtrlTiggaConfigFilter configFilter;
	ConfigFile* pConfigFileFilter;

	CtrlTiggaConfigPID configPid;
	ConfigFile* pConfigFilePid;

	CtrlTiggaConfigMisc configMisc;
	ConfigFile* pConfigFileMisc;

	LogCtrlTiggaDebug log;

	uint32_t lastVisionInputTime;
	uint8_t visionLost;
} CtrlTigga;

extern CtrlTigga ctrlTigga;
extern CtrlInstance ctrlTiggaInstance;

void CtrlTiggaInitCfg();
void CtrlTiggaSetVelTarget(float x, float y, float w);
void CtrlTiggaSetPosTarget(float x, float y, float w, float t);
void CtrlTiggaSetWheelVelocity(float fr, float fl, float rl, float rr);

#endif /* CTRL_TIGGA_H_ */
