function [ logData, in, out, sampleTimes, debug] = LogFileViewer()
%LOGFILEVIEWER Graphical viewer for log file data
%   Awesome tool for data analysis

clc;
clear;

global state;
global filename;
global pathname;
% [filename, pathname] = uigetfile('*.dat');
% logData = loadLogFile(fullfile(pathname, filename));

%filename = '../move_rotate.dat';
%filename = '../move_straight.dat';
%filename = '../move_side.dat';
%logData = loadLogFile(filename);

% [in, state, out, sampleTimes] = processCtrlData(logData);
% [ pid ] = processPidData( logData );

figure('Name', strcat('Logfile Viewer: ', filename), 'NumberTitle', 'Off');
m = uimenu('Label', 'Dataset');
uimenu(m, 'Label', 'Load File', 'Callback', @loadFile);
uimenu(m, 'Label', 'Global Position (X of Y)', 'Callback', @showGlobalPosition);
uimenu(m, 'Label', 'Global Position (XYW of t)', 'Callback', @showGlobalPositionXYW);
uimenu(m, 'Label', 'Local Position (Y of X)', 'Callback', @showLocalPositionXY);
uimenu(m, 'Label', 'Local Position (XYW of t)', 'Callback', @showLocalPositionXYW);
uimenu(m, 'Label', 'Vision Delta T', 'Callback', @showVisionDeltaT);
uimenu(m, 'Label', 'Local Velocity', 'Callback', @showLocalVelocity);
uimenu(m, 'Label', 'Local Velocity 2', 'Callback', @showLocalVelocity2);
uimenu(m, 'Label', 'Local Velocity 3', 'Callback', @showLocalVelocity3);
uimenu(m, 'Label', 'Motor Velocity', 'Callback', @showMotorVelocities);
uimenu(m, 'Label', 'Motor Voltages', 'Callback', @showMotorVoltages);
uimenu(m, 'Label', 'Acceleration', 'Callback', @showAcceleration);
uimenu(m, 'Label', 'Accel. Velocity', 'Callback', @showAccelerometerVelocity);
uimenu(m, 'Label', 'Jerk', 'Callback', @showJerk);
uimenu(m, 'Label', 'Gyro FFT', 'Callback', @showGyrFFT);
uimenu(m, 'Label', 'Acc. FFT', 'Callback', @showAccFFT);
ctrlLocal = uimenu(m, 'Label', 'Ctrl: Local Vel');
uimenu(ctrlLocal, 'Label', 'XY', 'Callback', @showVelCtrlXY);
ctrlGlobal = uimenu(m, 'Label', 'Ctrl: Global Pos');
uimenu(ctrlGlobal, 'Label', 'XY', 'Callback', @showPosCtrlXY);
uimenu(ctrlGlobal, 'Label', 'W', 'Callback', @showPosCtrlW);
uimenu(m, 'Label', 'Command Response', 'Callback', @showCmdResponse);
uimenu(m, 'Label', 'Slip', 'Callback', @showSlip);
uimenu(m, 'Label', 'Motor Vel Error', 'Callback', @showMotorVelocityError);


% uimenu(m, 'Label', 'PIDs', 'Callback', @showPID);
% uimenu(m, 'Label', 'trajectory', 'Callback', @showTrajectory);

loadFile(0,0);

function loadFile(~, ~)
    [filename, pathname] = uigetfile('*.dat');
    logData = loadLogFile(fullfile(pathname, filename));
    [in, state, out, sampleTimes] = processCtrlData(logData);
	[debug] = processDebugData(logData, sampleTimes);
%     [ pid ] = processPidData( logData );
    showLocalPositionXY(0, 0);
end

function showLocalPositionXY(~, ~)
    subplot(1, 1, 1);
    plot(in.pos.enc.local(:,1), in.pos.enc.local(:,2), ...
        in.pos.vis.local(:,1), in.pos.vis.local(:,2));
    axis equal
    legend('Encoder', 'Vision');
    title('Local Position (Vision+Encoder)');
    xlabel('X [m]');
    ylabel('Y [m]');
end

function showVisionDeltaT(~, ~)
    subplot(1, 1, 1);
    plot(sampleTimes, in.pos.vis.dt);
    title('Vision Delta Time');
    axis tight
    xlabel('t [s]');
    ylabel('t [s]');
end

function showLocalPositionXYW(~, ~)
	sp1 = subplot(2, 1, 1);
	plot(sampleTimes, in.pos.enc.local(:,1), 'b-', sampleTimes, in.pos.vis.local(:,1), 'c:', ...
		sampleTimes, state.pos.local(:,1), 'k-',...
		sampleTimes, in.pos.enc.local(:,2), 'r-', sampleTimes, in.pos.vis.local(:,2), 'm:', ...
		sampleTimes, state.pos.local(:,2), 'g-');
	legend('X Enc', 'X Vision', 'X State', 'Y Enc', 'Y Vision', 'Y State');
	title('XY Position (Vision+Encoders)');
	xlabel('t [s]');
	ylabel('s [m]');
	axis tight

    sp2 = subplot(2, 1, 2);
    plot(sampleTimes, in.pos.enc.local(:,3), 'b-', sampleTimes, in.pos.vis.local(:,3), 'r:', ...
		sampleTimes, in.pos.gyr.local(:,3), 'g-', sampleTimes, state.pos.local(:,3), 'k-')
    axis tight
    legend('Encoder', 'Vision', 'Gyro', 'State');
    title('Local Orientation');
    xlabel('t [s]');
    ylabel('w [rad]');
    
    linkaxes([sp1, sp2], 'x');
end

function showGlobalPositionXYW(~, ~)
	sp1 = subplot(2, 1, 1);
	plot(sampleTimes, in.pos.vis.global(:,1), 'b:', ...
		sampleTimes, state.pos.global(:,1), 'k-',...
		sampleTimes, in.pos.vis.global(:,2), 'm:', ...
		sampleTimes, state.pos.global(:,2), 'g-');
	legend('X Vision', 'X State', 'Y Vision', 'Y State');
	title('XY Position (Vision+State)');
	xlabel('t [s]');
	ylabel('s [m]');
	axis tight

    sp2 = subplot(2, 1, 2);
    plot(sampleTimes, in.pos.enc.local(:,3), 'b-', sampleTimes, in.pos.vis.global(:,3), 'r:', ...
		sampleTimes, in.pos.gyr.local(:,3), 'g-', sampleTimes, state.pos.global(:,3), 'k-')
    axis tight
    legend('Encoder', 'Vision', 'Gyro', 'State');
    title('Global Orientation');
    xlabel('t [s]');
    ylabel('w [rad]');
    
    linkaxes([sp1, sp2], 'x');
end

function showMotorVelocities(~, ~)
	vmin = min(min(in.mot.vel));
	vmax = max(max(in.mot.vel));
	vlim = [vmin vmax];
	
    sp1 = subplot(2, 2, 2);
    plot(sampleTimes, in.mot.vel(:,1), 'b-', sampleTimes, circshift(in.mot.vis.vel(:,1),-50), 'g--', ...
        sampleTimes, out.mot.vel(:,1), 'r-');
    legend('Encoder', 'Vision', 'Setpoint');
    title('M1 (Front Right)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
    grid minor
	ylim(vlim);    

    sp2 = subplot(2, 2, 1);
    plot(sampleTimes, in.mot.vel(:,2), 'b-', sampleTimes, circshift(in.mot.vis.vel(:,2),-50), 'g--', ...
        sampleTimes, out.mot.vel(:,2), 'r-');
    title('M2 (Front Left)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
    grid minor
	ylim(vlim);

    sp3 = subplot(2, 2, 3);
    plot(sampleTimes, in.mot.vel(:,3), 'b-', sampleTimes, circshift(in.mot.vis.vel(:,3),-50), 'g--', ...
        sampleTimes, out.mot.vel(:,3), 'r-');
    title('M3 (Rear Left)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
    grid minor
	ylim(vlim);

    sp4 = subplot(2, 2, 4);
    plot(sampleTimes, in.mot.vel(:,4), 'b-', sampleTimes, circshift(in.mot.vis.vel(:,4),-50), 'g--', ...
        sampleTimes, out.mot.vel(:,4), 'r-');
    title('M4 (Rear Right)');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
    grid minor
	ylim(vlim);
    
    linkaxes([sp1, sp2, sp3, sp4], 'x');
end

function showMotorVoltages(~, ~)
    bat = max(in.bat.vol);
    ulim = [-bat-1 bat+1];
    
    ke = 0.0251;
    
    sp1 = subplot(2, 2, 2);
    plot(sampleTimes, in.mot.vel(:,1)*ke, sampleTimes, out.mot.vol(:,1),...
        sampleTimes, in.bat.vol, sampleTimes, -in.bat.vol);
    legend('Back-EMF', 'Output');
    title('M1 (Front Right)');
    ylabel('U [V]');
    xlabel('t [s]');
    grid on;
    axis tight;
	ylim(ulim);    

    sp2 = subplot(2, 2, 1);
    plot(sampleTimes, in.mot.vel(:,2)*ke, sampleTimes, out.mot.vol(:,2),...
        sampleTimes, in.bat.vol, sampleTimes, -in.bat.vol);
    title('M2 (Front Left)');
    ylabel('U [V]');
    xlabel('t [s]');
    grid on;
	axis tight
	ylim(ulim);

    sp3 = subplot(2, 2, 3);
    plot(sampleTimes, in.mot.vel(:,3)*ke, sampleTimes, out.mot.vol(:,3),...
        sampleTimes, in.bat.vol, sampleTimes, -in.bat.vol);
    title('M3 (Rear Left)');
    ylabel('U [V]');
    xlabel('t [s]');
    grid on;
	axis tight
	ylim(ulim);

    sp4 = subplot(2, 2, 4);
    plot(sampleTimes, in.mot.vel(:,4)*ke, sampleTimes, out.mot.vol(:,4),...
        sampleTimes, in.bat.vol, sampleTimes, -in.bat.vol);
    title('M4 (Rear Right)');
    ylabel('U [V]');
    xlabel('t [s]');
    grid on;
	axis tight
	ylim(ulim);
    
    linkaxes([sp1, sp2, sp3, sp4], 'x');
end

function showGyrFFT(~, ~)
    subplot(2, 1, 1);
    plot(logData.gyro_raw.data(:,1), logData.gyro_raw.data(:,4))
    title('Gyro Raw Data');
    xlabel('t [s]');
    ylabel('Raw');
	axis tight

    g = logData.gyro_raw.data;

    Fs = 800;
    L = length(g);

    NFFT = 2^nextpow2(L);
    Yx = fft(g(:,4), NFFT)/L;
    f = Fs/2*linspace(0,1,NFFT/2+1);

    absYx = 2*abs(Yx(1:NFFT/2+1));

    subplot(2, 1, 2);
    plot(f(10:end), absYx(10:end));
    title('FFT Analysis');
    xlabel('F [Hz]');
    ylabel('Amplitude');
	axis tight
end

function showAccFFT(~, ~)
    subplot(2, 1, 1);
    plot(logData.acc_raw.data(:,1), logData.acc_raw.data(:,4))
    title('Accelerometer Raw Data');
    xlabel('t [s]');
    ylabel('Raw');
	axis tight

    g = logData.acc_raw.data;

    Fs = 800;
    L = length(g);

    NFFT = 2^nextpow2(L);
    Yx = fft(g(:,4), NFFT)/L;
    f = Fs/2*linspace(0,1,NFFT/2+1);

    absYx = 2*abs(Yx(1:NFFT/2+1));

    subplot(2, 1, 2);
    plot(f(10:end), absYx(10:end));
    title('FFT Analysis');
    xlabel('F [Hz]');
    ylabel('Amplitude');
	axis tight
end

function showLocalVelocity(~, ~)
	sp1 = subplot(2, 1, 1);
	plot(sampleTimes, in.vel.enc.local(:,1), 'b-', sampleTimes, out.vel.enc.local(:,1), 'c:', ...
		sampleTimes, in.vel.enc.local(:,2), 'r-', sampleTimes, out.vel.enc.local(:,2), 'm:', ...
        sampleTimes, in.vel.vis.local(:,1), 'g--', sampleTimes, in.vel.vis.local(:,2), 'k--');
	legend('X Enc', 'X Setpoint', 'Y Enc', 'Y Setpoint', 'X Vision', 'Y Vision');
	title('XY Velocities (Encoders+Vision)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight

	sp2 = subplot(2, 1, 2);
	plot(sampleTimes, in.vel.enc.local(:,3), 'g-', ...
		sampleTimes, smooth(in.vel.gyr.local(:,3), 1), 'm-', ...
		sampleTimes, out.vel.enc.local(:,3), 'k:', ...
        sampleTimes, in.vel.vis.local(:,3), 'c--');
	legend('W Enc', 'W Gyro', 'W Setpoint', 'W Vision');
	title('Rotational Velocity (Encoders+Gyro+Vision)');
	xlabel('t [s]');
	ylabel('v [rad/s]');
	axis tight
    
    linkaxes([sp1, sp2], 'x');
end

function showLocalVelocity2(~, ~)
    sp1 = subplot(2, 2, 1);
	plot(sampleTimes, in.vel.enc.local(:,1).*debug.enc2real(:,1), 'g--', ...
        sampleTimes, in.vel.enc.local(:,1), 'k:', ...
        sampleTimes, circshift(smooth(in.vel.vis.local(:,1),20),-20), 'c-', ...
        sampleTimes, debug.traj.vel.local(:,1), 'r:', ...
        sampleTimes, state.vel.local(:,1), 'b-');
	legend('Enc Cor.', 'Enc Raw', 'Vision', 'Traj', 'State');
	title('X Velocities (Encoders+Accelerometer+Vision)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight
    grid on
    grid minor
	ylim([-5 5]);
    
	sp2 = subplot(2, 2, 2);
	plot(sampleTimes, in.vel.enc.local(:,2).*debug.enc2real(:,2), 'g--', ...
        sampleTimes, in.vel.enc.local(:,2), 'k:', ...
        sampleTimes, in.vel.vis.local(:,2), 'c-', ...
        sampleTimes, debug.traj.vel.local(:,2), 'r:', ...
        sampleTimes, state.vel.local(:,2), 'b-');
    legend('Enc Cor.', 'Enc Raw', 'Vision', 'Traj', 'State');
	title('Y Velocities (Encoders+Accelerometer+Vision)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight
    grid on
    grid minor
	ylim([-5 5]);

	sp3 = subplot(2, 2, 3);
	plot(sampleTimes, in.vel.enc.local(:,3), 'g--', ...
		sampleTimes, in.vel.gyr.local(:,3), 'm--', ...
        sampleTimes, in.vel.vis.local(:,3), 'c-', ...
        sampleTimes, state.vel.local(:,3), 'b-');
	legend('Enc', 'Gyro', 'Vision', 'State');
	title('Rotational Velocity (Encoders+Gyro+Vision)');
	xlabel('t [s]');
	ylabel('v [rad/s]');
	axis tight
    grid on
    grid minor
	ylim([-10 10]);
    
	sp4 = subplot(2, 2, 4);
	plot(sampleTimes, in.vel.enc.local(:,4), 'g--', ...
        sampleTimes, in.vel.vis.local(:,4), 'c-', ...
        sampleTimes, state.vel.local(:,4), 'b-');
	legend('Enc', 'Vision', 'State');
	title('Norm(XY) Velocity (Encoders+Gyro+Vision)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight
    grid on
    grid minor
	ylim([0 5]);
    
    linkaxes([sp1, sp2, sp3, sp4], 'x');
end

function showLocalVelocity3(~, ~)
	sp1 = subplot(2, 1, 1);
	plot(sampleTimes, in.vel.enc.local(:,1), 'b-', sampleTimes, in.opt.vel(:,1), 'c:', ...
		sampleTimes, in.vel.enc.local(:,2), 'r-', sampleTimes, in.opt.vel(:,2), 'm:', ...
        sampleTimes, in.vel.vis.local(:,1), 'g--', sampleTimes, in.vel.vis.local(:,2), 'k--');
	legend('X Enc', 'X Optical', 'Y Enc', 'Y Optical', 'X Vision', 'Y Vision');
	title('XY Velocities (Encoders+Vision)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight

	sp2 = subplot(2, 1, 2);
	plot(sampleTimes, in.vel.enc.local(:,3), 'g-', ...
		sampleTimes, smooth(in.vel.gyr.local(:,3), 1), 'm-', ...
		sampleTimes, in.opt.vel(:,3), 'k:', ...
        sampleTimes, in.vel.vis.local(:,3), 'c--');
	legend('W Enc', 'W Gyro', 'W Optical', 'W Vision');
	title('Rotational Velocity (Encoders+Gyro+Vision)');
	xlabel('t [s]');
	ylabel('v [rad/s]');
	axis tight
    
    linkaxes([sp1, sp2], 'x');
end

function showGlobalPosition(~, ~)
    subplot(1, 1, 1);
    plot(in.pos.vis.global(:,1), in.pos.vis.global(:,2), ...
		state.pos.global(:,1), state.pos.global(:,2),...
		in.pos.vis.global(1,1), in.pos.vis.global(1,2), 'x',...
		state.pos.global(1,1), state.pos.global(1,2), 'x');
    hold on;
    qiv = 1:50:length(sampleTimes);
    quiver(in.pos.vis.global(qiv,1), in.pos.vis.global(qiv,2), ...
        cos(in.pos.vis.global(qiv,3)), sin(in.pos.vis.global(qiv,3)), 'g--');
    hold off;
	axis equal
    legend('Vision', 'State');
    title('Global Position (Vision+State)');
    xlabel('X [m]');
    ylabel('Y [m]');
    
end

function showAcceleration(~, ~)
    subplot(1, 1, 1);
    plot(sampleTimes, smooth(in.acc.acc.local(:,1), 50), 'b-', sampleTimes, smooth(in.acc.acc.local(:,2), 1), 'r-', ...
        sampleTimes, smooth(in.acc.acc.local(:,3), 1), 'g-', ...
		sampleTimes, in.acc.vis.local(:,1), 'c:', sampleTimes, in.acc.vis.local(:,2), 'm:', ...
        sampleTimes, debug.traj.acc.local(:,1), 'b--', sampleTimes, debug.traj.acc.local(:,2), 'r--');
    axis tight
    legend('X Acc', 'Y Acc', 'Z Acc', 'X Vision', 'Y Vision', 'X Traj', 'Y Traj');
    title('Acceleration');
    xlabel('t [s]');
    ylabel('a [m/s^2]');
end

function showJerk(~, ~)
    subplot(1, 1, 1);
    plot(sampleTimes, smooth(in.jerk.acc.local(:,1), 100), 'b-', sampleTimes, smooth(in.jerk.acc.local(:,2), 100), 'r-', ...
        sampleTimes, debug.traj.acc.local(:,1), 'b--', sampleTimes, debug.traj.acc.local(:,2), 'r--');
    axis tight
    legend('X Acc', 'Y Acc', 'X Traj', 'Y Traj');
    title('Jerk');
    xlabel('t [s]');
    ylabel('j [m/s^3]');
end

function showAccelerometerVelocity(~, ~)
    subplot(1, 1, 1);
    plot(sampleTimes, in.vel.acc.local(:,1), sampleTimes, in.vel.acc.local(:,2));
    axis tight
    legend('X', 'Y');
    title('Integrated Accelerometer Velocity');
    xlabel('t [s]');
    ylabel('v [m/s]');
end

function showVelCtrlXY(~, ~)
	sp1 = subplot(2, 2, 1);
	plot(sampleTimes, state.vel.local(:,1), 'g-', sampleTimes, state.vel.local(:,2), 'k-', ...
        sampleTimes, in.vel.vis.local(:,1), 'g:', sampleTimes, in.vel.vis.local(:,2), 'k:', ...
		sampleTimes, debug.traj.vel.local(:,1), 'c-', sampleTimes, debug.traj.vel.local(:,2), 'm-', ...
		sampleTimes, debug.set.vel.local(:,1), 'b-', sampleTimes, debug.set.vel.local(:,2), 'r-');
	legend('X State', 'Y State', 'X Vision', 'Y Vision', 'X Ref', 'Y Ref', 'X Set', 'Y Set');
	title('XY Velocities (State+Vision+VelInt+Setpoint)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight

	sp3 = subplot(2, 2, 3);
	plot(sampleTimes, state.vel.local(:,3), 'k:', ...
        sampleTimes, in.vel.vis.local(:,3), 'c--', ...
		sampleTimes, debug.traj.vel.local(:,3), 'g-', ...
		sampleTimes, debug.set.vel.local(:,3), 'r-');
	legend('W State', 'W Vision', 'W Ref', 'W Setpoint');
	title('Rotational Velocity (State+Vision+VelInt+Setpoint)');
	xlabel('t [s]');
	ylabel('v [rad/s]');
	axis tight

	sp2 = subplot(2, 2, 2);
	plot(sampleTimes, debug.traj.acc.local(:,1), 'k:', ...
		sampleTimes, state.vel.local(:,1), 'g-', ...
		sampleTimes, debug.set.vel.local(:,1), 'r-');
	legend('X Acc', 'X Vel State', 'X Vel Ref');
	title('X Control');
	xlabel('t [s]');
	ylabel('v [rad/s]');
	axis tight

	sp4 = subplot(2, 2, 4);
	plot(sampleTimes, debug.ctrlMode(:,1), 'k-');
	title('Ctrl Mode');
	xlabel('t [s]');
	ylabel('Mode');
	axis tight
	ylim([-1 2]);
    
    linkaxes([sp1, sp2, sp3, sp4], 'x');
end

function showPosCtrlXY(~, ~)
	sp1 = subplot(2, 2, 1);
	plot(sampleTimes, state.pos.global(:,1), 'b-', ...
        sampleTimes, in.pos.vis.global(:,1), 'r:', ...
		sampleTimes, debug.set.pos.global(:,1), 'g--', ...
        sampleTimes, debug.traj.pos.global(:,1), 'm--');
	legend('State', 'Vision', 'Set', 'Traj');
	title('X Position');
	xlabel('t [s]');
	ylabel('p [m]');
	axis tight
    grid minor

	sp2 = subplot(2, 2, 2);
	plot(sampleTimes, state.pos.global(:,2), 'b-', ...
        sampleTimes, in.pos.vis.global(:,2), 'r:', ...
		sampleTimes, debug.set.pos.global(:,2), 'g--', ...
        sampleTimes, debug.traj.pos.global(:,2), 'm--');
	legend('State', 'Vision', 'Set', 'Traj');
	title('Y Position');
	xlabel('t [s]');
	ylabel('p [m]');
	axis tight
	grid minor

	sp3 = subplot(2, 2, 3);
	plot(sampleTimes, state.vel.global(:,1), 'b-', ...
        sampleTimes, debug.traj.vel.global(:,1), 'm--');
    legend('State', 'Traj');
	title('X Velocity');
	xlabel('t [s]');
	axis tight
    grid minor

	sp4 = subplot(2, 2, 4);
    plot(sampleTimes, state.vel.global(:,2), 'b-', ...
        sampleTimes, debug.traj.vel.global(:,2), 'm--');
    legend('State', 'Traj');
	title('Y Velocity');
	xlabel('t [s]');
	axis tight
    grid minor
	
	linkaxes([sp1, sp2, sp3, sp4], 'x');
end

function showPosCtrlW(~, ~)
	sp1 = subplot(2, 2, 1);
	plot(sampleTimes, state.pos.global(:,3), 'b-', ...
        sampleTimes, in.pos.vis.global(:,3), 'r:', ...
		sampleTimes, debug.set.pos.global(:,3), 'g--', ...
        sampleTimes, debug.traj.pos.global(:,3), 'm--');
	legend('W State', 'W Vision', 'W Set', 'W Traj');
	title('W Orientation (State+Vision+Setpoint)');
	xlabel('t [s]');
	ylabel('p [rad]');
	axis tight
    grid minor

	sp2 = subplot(2, 2, 3);
    plot(sampleTimes, state.vel.global(:,3), 'b-', ...
        sampleTimes, in.vel.gyr.local(:,3), 'k:', ...
        sampleTimes, debug.traj.vel.global(:,3), 'm--');
    legend('State', 'Gyro', 'Traj');
	title('W Velocity');
	xlabel('t [s]');
	axis tight
    grid minor

	sp3 = subplot(2, 2, 2);
	plot(sampleTimes, debug.time.fusion(:,1), 'k-', ...
		sampleTimes, debug.time.ctrl(:,1), 'g-');
	legend('Fusion Time', 'Ctrl Time');
	title('Processing Time');
	xlabel('t [s]');
	ylabel('t [us]');
	axis tight
	ylim([0 1000]);

	sp4 = subplot(2, 2, 4);
	plot(sampleTimes, debug.ctrlMode(:,1), 'k-');
	title('Ctrl Mode');
	xlabel('t [s]');
	ylabel('Mode');
	axis tight
	ylim([-1 2]);
	
	linkaxes([sp1, sp3, sp4, sp2], 'x');
end

function showCmdResponse(~, ~)
    subplot(1, 1, 1);
    plot(sampleTimes, out.mot.vol(:,1), 'b-', sampleTimes, in.pos.vis.global(:,2), 'x', ...
		sampleTimes, in.pos.vis.global(:,1), 'o');
    axis tight
    legend('M1 Voltage', 'X Vision', 'Y Vision');
    title('Command response timing');
    xlabel('t [s]');
    ylabel('a [m/s^2]');
	grid on;
	grid minor;
end

function showSlip(~, ~)
	sp1 = subplot(2, 1, 1);
	plot(sampleTimes, in.vel.enc.local(:,4), 'b-');
% 	legend('X Enc', 'Y Enc');
	title('XY Norm Velocity (Encoders)');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight

	sp2 = subplot(2, 1, 2);
	plot(sampleTimes, in.slip.enc(:,1), 'g-');
% 	legend('Slip');
	title('Slip');
	xlabel('t [s]');
	ylabel('v [m/s]');
	axis tight
    
    linkaxes([sp1, sp2], 'x');
end

function showMotorVelocityError(~, ~)
    driveAngle = atan2(in.vel.vis.local(:,2), in.vel.vis.local(:,1))*180/pi;
    
    sp1 = subplot(2, 1, 1);
    plot(sampleTimes, driveAngle);
    legend('Vision');
    title('Local Driving Angle');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
    grid minor
	ylim([-190 190]);    

    sp2 = subplot(2, 1, 2);
    plot(sampleTimes, smooth(in.mot.vis.vel(:,1)-out.mot.vel(:,1),50), 'b-', ...
        sampleTimes, smooth(in.mot.vis.vel(:,2)-out.mot.vel(:,2),50), 'r-', ...
        sampleTimes, smooth(in.mot.vis.vel(:,3)-out.mot.vel(:,3),50), 'g-', ...
        sampleTimes, smooth(in.mot.vis.vel(:,4)-out.mot.vel(:,4),50), 'k-');
    legend('M1', 'M2', 'M3', 'M4');
    title('Motor Velocity Error');
    ylabel('v [Motor RPS]');
    xlabel('t [s]');
	axis tight
    grid minor
    linkaxes([sp1, sp2], 'x');
end

end
