function [ debug ] = processDebugData( logData, sampleTimes )
%PROCESSDEBUGDATA Summary of this function goes here
%   Detailed explanation goes here

numSamples = length(sampleTimes);
[debug.traj.pos.global, debug.traj.vel.global, debug.traj.acc.global, debug.traj.jerk.global, ...
    debug.traj.vel.local, debug.traj.acc.local, debug.traj.jerk.local] = procDebugTraj();
[debug.set.pos.global, debug.set.vel.local] = procDebugSetpoints();
debug.ctrlMode = procDebugMisc();
[debug.time.fusion, debug.time.ctrl] = procDebugTime();
debug.enc2real = procEnc2Real();

function [pos, velGlobal, accGlobal, jerkGlobal, velLocal, accLocal, jerkLocal] = procDebugTraj()
	firstCol = find(strcmp(logData.ctrl_tigga_debug.names, 'traj_pos_x'));

	trajData = logData.ctrl_tigga_debug.data(:,[1 firstCol:firstCol+11]);

    pos = zeros(numSamples, 3);
    velGlobal = zeros(numSamples, 3);
	accGlobal = zeros(numSamples, 3);
    jerkGlobal = zeros(numSamples, 3);
    velLocal = zeros(numSamples, 3);
	accLocal = zeros(numSamples, 3);
    jerkLocal = zeros(numSamples, 3);

	[~, ia] = unique(trajData(:,1));	% find unqiue entries
	trajData = trajData(ia, :);	% delete duplicates

	for i = 1:3
        pos(:,i) = interp1(trajData(:,1), trajData(:,i+1), sampleTimes);
        velLocal(:,i) = interp1(trajData(:,1), trajData(:,i+4), sampleTimes);
		accLocal(:,i) = interp1(trajData(:,1), trajData(:,i+7), sampleTimes);
        jerkLocal(:,i) = interp1(trajData(:,1), trajData(:,i+10), sampleTimes);
	end
    
    for i = 1:numSamples
        angle = -pi/2+pos(i,3);
		rotMat = [cos(angle) -sin(angle); sin(angle) cos(angle)];
        
		velGlobal(i,1:2) = (rotMat*(velLocal(i,1:2)'))';
        accGlobal(i,1:2) = (rotMat*(accLocal(i,1:2)'))';
        jerkGlobal(i,1:2) = (rotMat*(jerkLocal(i,1:2)'))';
    end
    
    velGlobal(:,3) = velLocal(:,3);
    accGlobal(:,3) = accLocal(:,3);
    jerkGlobal(:,3) = jerkLocal(:,3);
end

function [setPosGlobal, setVelLocal] = procDebugSetpoints()
	firstCol = find(strcmp(logData.ctrl_tigga_debug.names, 'set_pos_x'));

	setData = logData.ctrl_tigga_debug.data(:,[1 firstCol:firstCol+5]);

	setPosGlobal = zeros(numSamples, 3);
	setVelLocal = zeros(numSamples, 3);

	[~, ia] = unique(setData(:,1));	% find unqiue entries
	setData = setData(ia, :);	% delete duplicates

	for i = 1:3
		setPosGlobal(:,i) = interp1(setData(:,1), setData(:,i+1), sampleTimes);
		setVelLocal(:,i) = interp1(setData(:,1), setData(:,i+4), sampleTimes);
	end
end

function [ctrlMode] = procDebugMisc()
	firstCol = find(strcmp(logData.ctrl_tigga_debug.names, 'ctrl_mode'));

	ctrlModeData = logData.ctrl_tigga_debug.data(:,[1 firstCol]);

	ctrlMode = zeros(numSamples, 1);

	[~, ia] = unique(ctrlModeData(:,1));	% find unqiue entries
	ctrlModeData = ctrlModeData(ia, :);	% delete duplicates

	ctrlMode(:,1) = interp1(ctrlModeData(:,1), ctrlModeData(:,2), sampleTimes);
end

function [fusionTime, ctrlTime] = procDebugTime()
	firstCol = find(strcmp(logData.ctrl_tigga_debug.names, 'fusion_time'));

	timeData = logData.ctrl_tigga_debug.data(:,[1 firstCol firstCol+1]);

	ctrlTime = zeros(numSamples, 1);
	fusionTime = zeros(numSamples, 1);

	[~, ia] = unique(timeData(:,1));	% find unqiue entries
	timeData = timeData(ia, :);	% delete duplicates

	fusionTime(:,1) = interp1(timeData(:,1), timeData(:,2), sampleTimes);
	ctrlTime(:,1) = interp1(timeData(:,1), timeData(:,3), sampleTimes);
end

function [enc2real] = procEnc2Real()
    firstCol = find(strcmp(logData.ctrl_tigga_debug.names, 'enc2real_x'));

	enc2realData = logData.ctrl_tigga_debug.data(:,[1 firstCol firstCol+1]);

	enc2real = zeros(numSamples, 2);

	[~, ia] = unique(enc2realData(:,1));	% find unqiue entries
	enc2realData = enc2realData(ia, :);	% delete duplicates

	enc2real(:,1) = interp1(enc2realData(:,1), enc2realData(:,2), sampleTimes);
    enc2real(:,2) = interp1(enc2realData(:,1), enc2realData(:,3), sampleTimes);
end

end
